# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'nanogen.ui'
#
# Created: Sun Mar 16 13:07:48 2014
#      by: PyQt4 UI code generator 4.10.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_NanoGen(object):
    def setupUi(self, NanoGen):
        NanoGen.setObjectName(_fromUtf8("NanoGen"))
        NanoGen.resize(532, 729)
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Lucida Sans Unicode"))
        font.setPointSize(14)
        NanoGen.setFont(font)
        self.centralwidget = QtGui.QWidget(NanoGen)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.verticalLayout_12 = QtGui.QVBoxLayout(self.centralwidget)
        self.verticalLayout_12.setObjectName(_fromUtf8("verticalLayout_12"))
        self.nanogen_tabWidget = QtGui.QTabWidget(self.centralwidget)
        self.nanogen_tabWidget.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(20)
        self.nanogen_tabWidget.setFont(font)
        self.nanogen_tabWidget.setObjectName(_fromUtf8("nanogen_tabWidget"))
        self.nanotube_generator_tabWidget = QtGui.QWidget()
        self.nanotube_generator_tabWidget.setObjectName(_fromUtf8("nanotube_generator_tabWidget"))
        self.verticalLayout_10 = QtGui.QVBoxLayout(self.nanotube_generator_tabWidget)
        self.verticalLayout_10.setObjectName(_fromUtf8("verticalLayout_10"))
        self.horizontalLayout_38 = QtGui.QHBoxLayout()
        self.horizontalLayout_38.setObjectName(_fromUtf8("horizontalLayout_38"))
        self.verticalLayout_6 = QtGui.QVBoxLayout()
        self.verticalLayout_6.setObjectName(_fromUtf8("verticalLayout_6"))
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.nanotube_generator_radioButton = QtGui.QRadioButton(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.nanotube_generator_radioButton.setFont(font)
        self.nanotube_generator_radioButton.setChecked(True)
        self.nanotube_generator_radioButton.setObjectName(_fromUtf8("nanotube_generator_radioButton"))
        self.nanotube_generator_buttonGroup = QtGui.QButtonGroup(NanoGen)
        self.nanotube_generator_buttonGroup.setObjectName(_fromUtf8("nanotube_generator_buttonGroup"))
        self.nanotube_generator_buttonGroup.addButton(self.nanotube_generator_radioButton)
        self.verticalLayout_2.addWidget(self.nanotube_generator_radioButton)
        self.capped_nanotube_generator_radioButton = QtGui.QRadioButton(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.capped_nanotube_generator_radioButton.setFont(font)
        self.capped_nanotube_generator_radioButton.setCheckable(False)
        self.capped_nanotube_generator_radioButton.setObjectName(_fromUtf8("capped_nanotube_generator_radioButton"))
        self.nanotube_generator_buttonGroup.addButton(self.capped_nanotube_generator_radioButton)
        self.verticalLayout_2.addWidget(self.capped_nanotube_generator_radioButton)
        self.mwnt_generator_radioButton = QtGui.QRadioButton(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.mwnt_generator_radioButton.setFont(font)
        self.mwnt_generator_radioButton.setObjectName(_fromUtf8("mwnt_generator_radioButton"))
        self.nanotube_generator_buttonGroup.addButton(self.mwnt_generator_radioButton)
        self.verticalLayout_2.addWidget(self.mwnt_generator_radioButton)
        self.unrolled_nanotube_generator_radioButton = QtGui.QRadioButton(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.unrolled_nanotube_generator_radioButton.setFont(font)
        self.unrolled_nanotube_generator_radioButton.setObjectName(_fromUtf8("unrolled_nanotube_generator_radioButton"))
        self.nanotube_generator_buttonGroup.addButton(self.unrolled_nanotube_generator_radioButton)
        self.verticalLayout_2.addWidget(self.unrolled_nanotube_generator_radioButton)
        self.bundle_generator_checkBox = QtGui.QCheckBox(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.bundle_generator_checkBox.setFont(font)
        self.bundle_generator_checkBox.setCheckable(True)
        self.bundle_generator_checkBox.setObjectName(_fromUtf8("bundle_generator_checkBox"))
        self.verticalLayout_2.addWidget(self.bundle_generator_checkBox)
        self.verticalLayout_6.addLayout(self.verticalLayout_2)
        self.line = QtGui.QFrame(self.nanotube_generator_tabWidget)
        self.line.setFrameShape(QtGui.QFrame.HLine)
        self.line.setFrameShadow(QtGui.QFrame.Sunken)
        self.line.setObjectName(_fromUtf8("line"))
        self.verticalLayout_6.addWidget(self.line)
        self.horizontalLayout_13 = QtGui.QHBoxLayout()
        self.horizontalLayout_13.setObjectName(_fromUtf8("horizontalLayout_13"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.label_7 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(20)
        self.label_7.setFont(font)
        self.label_7.setObjectName(_fromUtf8("label_7"))
        self.horizontalLayout_2.addWidget(self.label_7)
        self.n_spinBox = QtGui.QSpinBox(self.nanotube_generator_tabWidget)
        self.n_spinBox.setMinimumSize(QtCore.QSize(90, 0))
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(20)
        self.n_spinBox.setFont(font)
        self.n_spinBox.setMaximum(100)
        self.n_spinBox.setProperty("value", 10)
        self.n_spinBox.setObjectName(_fromUtf8("n_spinBox"))
        self.horizontalLayout_2.addWidget(self.n_spinBox)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.label_14 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(20)
        self.label_14.setFont(font)
        self.label_14.setObjectName(_fromUtf8("label_14"))
        self.horizontalLayout.addWidget(self.label_14)
        self.m_spinBox = QtGui.QSpinBox(self.nanotube_generator_tabWidget)
        self.m_spinBox.setMinimumSize(QtCore.QSize(90, 0))
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(20)
        self.m_spinBox.setFont(font)
        self.m_spinBox.setMaximum(100)
        self.m_spinBox.setProperty("value", 10)
        self.m_spinBox.setObjectName(_fromUtf8("m_spinBox"))
        self.horizontalLayout.addWidget(self.m_spinBox)
        self.horizontalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_13.addLayout(self.horizontalLayout_3)
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem)
        self.verticalLayout_6.addLayout(self.horizontalLayout_13)
        self.line_2 = QtGui.QFrame(self.nanotube_generator_tabWidget)
        self.line_2.setFrameShape(QtGui.QFrame.HLine)
        self.line_2.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_2.setObjectName(_fromUtf8("line_2"))
        self.verticalLayout_6.addWidget(self.line_2)
        self.horizontalLayout_31 = QtGui.QHBoxLayout()
        self.horizontalLayout_31.setObjectName(_fromUtf8("horizontalLayout_31"))
        self.horizontalLayout_30 = QtGui.QHBoxLayout()
        self.horizontalLayout_30.setObjectName(_fromUtf8("horizontalLayout_30"))
        self.horizontalLayout_29 = QtGui.QHBoxLayout()
        self.horizontalLayout_29.setObjectName(_fromUtf8("horizontalLayout_29"))
        self.label_17 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(18)
        self.label_17.setFont(font)
        self.label_17.setObjectName(_fromUtf8("label_17"))
        self.horizontalLayout_29.addWidget(self.label_17)
        self.nanotube_element1_comboBox = QtGui.QComboBox(self.nanotube_generator_tabWidget)
        self.nanotube_element1_comboBox.setObjectName(_fromUtf8("nanotube_element1_comboBox"))
        self.nanotube_element1_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element1_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element1_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element1_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element1_comboBox.addItem(_fromUtf8(""))
        self.horizontalLayout_29.addWidget(self.nanotube_element1_comboBox)
        self.horizontalLayout_30.addLayout(self.horizontalLayout_29)
        self.horizontalLayout_28 = QtGui.QHBoxLayout()
        self.horizontalLayout_28.setObjectName(_fromUtf8("horizontalLayout_28"))
        self.label_18 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(18)
        self.label_18.setFont(font)
        self.label_18.setObjectName(_fromUtf8("label_18"))
        self.horizontalLayout_28.addWidget(self.label_18)
        self.nanotube_element2_comboBox = QtGui.QComboBox(self.nanotube_generator_tabWidget)
        self.nanotube_element2_comboBox.setObjectName(_fromUtf8("nanotube_element2_comboBox"))
        self.nanotube_element2_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element2_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element2_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element2_comboBox.addItem(_fromUtf8(""))
        self.nanotube_element2_comboBox.addItem(_fromUtf8(""))
        self.horizontalLayout_28.addWidget(self.nanotube_element2_comboBox)
        self.horizontalLayout_30.addLayout(self.horizontalLayout_28)
        self.horizontalLayout_31.addLayout(self.horizontalLayout_30)
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_31.addItem(spacerItem1)
        self.verticalLayout_6.addLayout(self.horizontalLayout_31)
        self.horizontalLayout_24 = QtGui.QHBoxLayout()
        self.horizontalLayout_24.setObjectName(_fromUtf8("horizontalLayout_24"))
        self.horizontalLayout_14 = QtGui.QHBoxLayout()
        self.horizontalLayout_14.setObjectName(_fromUtf8("horizontalLayout_14"))
        self.nanotube_e1e2_bond_label = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.nanotube_e1e2_bond_label.setFont(font)
        self.nanotube_e1e2_bond_label.setObjectName(_fromUtf8("nanotube_e1e2_bond_label"))
        self.horizontalLayout_14.addWidget(self.nanotube_e1e2_bond_label)
        self.nanotube_bond_doubleSpinBox = QtGui.QDoubleSpinBox(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nanotube_bond_doubleSpinBox.sizePolicy().hasHeightForWidth())
        self.nanotube_bond_doubleSpinBox.setSizePolicy(sizePolicy)
        self.nanotube_bond_doubleSpinBox.setMinimumSize(QtCore.QSize(150, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.nanotube_bond_doubleSpinBox.setFont(font)
        self.nanotube_bond_doubleSpinBox.setInputMethodHints(QtCore.Qt.ImhFormattedNumbersOnly)
        self.nanotube_bond_doubleSpinBox.setReadOnly(False)
        self.nanotube_bond_doubleSpinBox.setDecimals(3)
        self.nanotube_bond_doubleSpinBox.setMinimum(0.5)
        self.nanotube_bond_doubleSpinBox.setMaximum(10.0)
        self.nanotube_bond_doubleSpinBox.setSingleStep(0.01)
        self.nanotube_bond_doubleSpinBox.setProperty("value", 1.42)
        self.nanotube_bond_doubleSpinBox.setObjectName(_fromUtf8("nanotube_bond_doubleSpinBox"))
        self.horizontalLayout_14.addWidget(self.nanotube_bond_doubleSpinBox)
        self.horizontalLayout_24.addLayout(self.horizontalLayout_14)
        spacerItem2 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_24.addItem(spacerItem2)
        self.verticalLayout_6.addLayout(self.horizontalLayout_24)
        self.line_8 = QtGui.QFrame(self.nanotube_generator_tabWidget)
        self.line_8.setFrameShape(QtGui.QFrame.HLine)
        self.line_8.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_8.setObjectName(_fromUtf8("line_8"))
        self.verticalLayout_6.addWidget(self.line_8)
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.horizontalLayout_11 = QtGui.QHBoxLayout()
        self.horizontalLayout_11.setObjectName(_fromUtf8("horizontalLayout_11"))
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.label = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label.setFont(font)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout_4.addWidget(self.label)
        self.nx_spinBox = QtGui.QSpinBox(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nx_spinBox.sizePolicy().hasHeightForWidth())
        self.nx_spinBox.setSizePolicy(sizePolicy)
        self.nx_spinBox.setMinimumSize(QtCore.QSize(100, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.nx_spinBox.setFont(font)
        self.nx_spinBox.setMinimum(1)
        self.nx_spinBox.setObjectName(_fromUtf8("nx_spinBox"))
        self.horizontalLayout_4.addWidget(self.nx_spinBox)
        self.horizontalLayout_11.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_7 = QtGui.QHBoxLayout()
        self.horizontalLayout_7.setObjectName(_fromUtf8("horizontalLayout_7"))
        self.label_2 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_2.setFont(font)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.horizontalLayout_7.addWidget(self.label_2)
        self.Lx_lineEdit = QtGui.QLineEdit(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Lx_lineEdit.sizePolicy().hasHeightForWidth())
        self.Lx_lineEdit.setSizePolicy(sizePolicy)
        self.Lx_lineEdit.setMinimumSize(QtCore.QSize(175, 0))
        self.Lx_lineEdit.setMaximumSize(QtCore.QSize(200, 16777215))
        self.Lx_lineEdit.setReadOnly(True)
        self.Lx_lineEdit.setObjectName(_fromUtf8("Lx_lineEdit"))
        self.horizontalLayout_7.addWidget(self.Lx_lineEdit)
        self.horizontalLayout_11.addLayout(self.horizontalLayout_7)
        spacerItem3 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_11.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_10 = QtGui.QHBoxLayout()
        self.horizontalLayout_10.setObjectName(_fromUtf8("horizontalLayout_10"))
        self.horizontalLayout_5 = QtGui.QHBoxLayout()
        self.horizontalLayout_5.setObjectName(_fromUtf8("horizontalLayout_5"))
        self.label_3 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_3.setFont(font)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.horizontalLayout_5.addWidget(self.label_3)
        self.ny_spinBox = QtGui.QSpinBox(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ny_spinBox.sizePolicy().hasHeightForWidth())
        self.ny_spinBox.setSizePolicy(sizePolicy)
        self.ny_spinBox.setMinimumSize(QtCore.QSize(100, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.ny_spinBox.setFont(font)
        self.ny_spinBox.setMinimum(1)
        self.ny_spinBox.setObjectName(_fromUtf8("ny_spinBox"))
        self.horizontalLayout_5.addWidget(self.ny_spinBox)
        self.horizontalLayout_10.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_8 = QtGui.QHBoxLayout()
        self.horizontalLayout_8.setObjectName(_fromUtf8("horizontalLayout_8"))
        self.label_12 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_12.setFont(font)
        self.label_12.setObjectName(_fromUtf8("label_12"))
        self.horizontalLayout_8.addWidget(self.label_12)
        self.Ly_lineEdit = QtGui.QLineEdit(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Ly_lineEdit.sizePolicy().hasHeightForWidth())
        self.Ly_lineEdit.setSizePolicy(sizePolicy)
        self.Ly_lineEdit.setMinimumSize(QtCore.QSize(175, 0))
        self.Ly_lineEdit.setMaximumSize(QtCore.QSize(200, 16777215))
        self.Ly_lineEdit.setReadOnly(True)
        self.Ly_lineEdit.setObjectName(_fromUtf8("Ly_lineEdit"))
        self.horizontalLayout_8.addWidget(self.Ly_lineEdit)
        self.horizontalLayout_10.addLayout(self.horizontalLayout_8)
        spacerItem4 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_10.addItem(spacerItem4)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_12 = QtGui.QHBoxLayout()
        self.horizontalLayout_12.setObjectName(_fromUtf8("horizontalLayout_12"))
        self.horizontalLayout_6 = QtGui.QHBoxLayout()
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        self.label_4 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_4.setFont(font)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.horizontalLayout_6.addWidget(self.label_4)
        self.nz_spinBox = QtGui.QSpinBox(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.MinimumExpanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.nz_spinBox.sizePolicy().hasHeightForWidth())
        self.nz_spinBox.setSizePolicy(sizePolicy)
        self.nz_spinBox.setMinimumSize(QtCore.QSize(100, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.nz_spinBox.setFont(font)
        self.nz_spinBox.setMinimum(1)
        self.nz_spinBox.setMaximum(1000)
        self.nz_spinBox.setProperty("value", 1000)
        self.nz_spinBox.setObjectName(_fromUtf8("nz_spinBox"))
        self.horizontalLayout_6.addWidget(self.nz_spinBox)
        self.horizontalLayout_12.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_9 = QtGui.QHBoxLayout()
        self.horizontalLayout_9.setObjectName(_fromUtf8("horizontalLayout_9"))
        self.label_13 = QtGui.QLabel(self.nanotube_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_13.setFont(font)
        self.label_13.setObjectName(_fromUtf8("label_13"))
        self.horizontalLayout_9.addWidget(self.label_13)
        self.Lz_doubleSpinBox = QtGui.QDoubleSpinBox(self.nanotube_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Lz_doubleSpinBox.sizePolicy().hasHeightForWidth())
        self.Lz_doubleSpinBox.setSizePolicy(sizePolicy)
        self.Lz_doubleSpinBox.setMinimumSize(QtCore.QSize(200, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.Lz_doubleSpinBox.setFont(font)
        self.Lz_doubleSpinBox.setDecimals(4)
        self.Lz_doubleSpinBox.setMaximum(1000.0)
        self.Lz_doubleSpinBox.setObjectName(_fromUtf8("Lz_doubleSpinBox"))
        self.horizontalLayout_9.addWidget(self.Lz_doubleSpinBox)
        self.horizontalLayout_12.addLayout(self.horizontalLayout_9)
        spacerItem5 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem5)
        self.verticalLayout.addLayout(self.horizontalLayout_12)
        self.verticalLayout_6.addLayout(self.verticalLayout)
        self.horizontalLayout_38.addLayout(self.verticalLayout_6)
        spacerItem6 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_38.addItem(spacerItem6)
        self.verticalLayout_10.addLayout(self.horizontalLayout_38)
        self.nanogen_tabWidget.addTab(self.nanotube_generator_tabWidget, _fromUtf8(""))
        self.graphene_generator_tabWidget = QtGui.QWidget()
        self.graphene_generator_tabWidget.setObjectName(_fromUtf8("graphene_generator_tabWidget"))
        self.verticalLayout_8 = QtGui.QVBoxLayout(self.graphene_generator_tabWidget)
        self.verticalLayout_8.setObjectName(_fromUtf8("verticalLayout_8"))
        self.horizontalLayout_37 = QtGui.QHBoxLayout()
        self.horizontalLayout_37.setObjectName(_fromUtf8("horizontalLayout_37"))
        self.verticalLayout_7 = QtGui.QVBoxLayout()
        self.verticalLayout_7.setObjectName(_fromUtf8("verticalLayout_7"))
        spacerItem7 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_7.addItem(spacerItem7)
        self.line_10 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_10.setFrameShape(QtGui.QFrame.HLine)
        self.line_10.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_10.setObjectName(_fromUtf8("line_10"))
        self.verticalLayout_7.addWidget(self.line_10)
        self.horizontalLayout_20 = QtGui.QHBoxLayout()
        self.horizontalLayout_20.setObjectName(_fromUtf8("horizontalLayout_20"))
        self.horizontalLayout_19 = QtGui.QHBoxLayout()
        self.horizontalLayout_19.setObjectName(_fromUtf8("horizontalLayout_19"))
        self.verticalLayout_9 = QtGui.QVBoxLayout()
        self.verticalLayout_9.setObjectName(_fromUtf8("verticalLayout_9"))
        self.horizontalLayout_16 = QtGui.QHBoxLayout()
        self.horizontalLayout_16.setObjectName(_fromUtf8("horizontalLayout_16"))
        self.label_9 = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_9.setFont(font)
        self.label_9.setObjectName(_fromUtf8("label_9"))
        self.horizontalLayout_16.addWidget(self.label_9)
        self.length_doubleSpinBox = QtGui.QDoubleSpinBox(self.graphene_generator_tabWidget)
        self.length_doubleSpinBox.setMinimumSize(QtCore.QSize(200, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.length_doubleSpinBox.setFont(font)
        self.length_doubleSpinBox.setDecimals(4)
        self.length_doubleSpinBox.setMaximum(1000.0)
        self.length_doubleSpinBox.setProperty("value", 10.0)
        self.length_doubleSpinBox.setObjectName(_fromUtf8("length_doubleSpinBox"))
        self.horizontalLayout_16.addWidget(self.length_doubleSpinBox)
        self.verticalLayout_9.addLayout(self.horizontalLayout_16)
        self.horizontalLayout_17 = QtGui.QHBoxLayout()
        self.horizontalLayout_17.setObjectName(_fromUtf8("horizontalLayout_17"))
        self.label_10 = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_10.setFont(font)
        self.label_10.setObjectName(_fromUtf8("label_10"))
        self.horizontalLayout_17.addWidget(self.label_10)
        self.width_doubleSpinBox = QtGui.QDoubleSpinBox(self.graphene_generator_tabWidget)
        self.width_doubleSpinBox.setMinimumSize(QtCore.QSize(200, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.width_doubleSpinBox.setFont(font)
        self.width_doubleSpinBox.setDecimals(4)
        self.width_doubleSpinBox.setMaximum(1000.0)
        self.width_doubleSpinBox.setProperty("value", 1.0)
        self.width_doubleSpinBox.setObjectName(_fromUtf8("width_doubleSpinBox"))
        self.horizontalLayout_17.addWidget(self.width_doubleSpinBox)
        self.verticalLayout_9.addLayout(self.horizontalLayout_17)
        self.horizontalLayout_19.addLayout(self.verticalLayout_9)
        self.line_5 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_5.setFrameShape(QtGui.QFrame.VLine)
        self.line_5.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_5.setObjectName(_fromUtf8("line_5"))
        self.horizontalLayout_19.addWidget(self.line_5)
        self.horizontalLayout_20.addLayout(self.horizontalLayout_19)
        self.verticalLayout_3 = QtGui.QVBoxLayout()
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.label_5 = QtGui.QLabel(self.graphene_generator_tabWidget)
        self.label_5.setObjectName(_fromUtf8("label_5"))
        self.verticalLayout_3.addWidget(self.label_5)
        self.AC_edge_radioButton = QtGui.QRadioButton(self.graphene_generator_tabWidget)
        self.AC_edge_radioButton.setChecked(True)
        self.AC_edge_radioButton.setObjectName(_fromUtf8("AC_edge_radioButton"))
        self.Lx_edge_buttonGroup = QtGui.QButtonGroup(NanoGen)
        self.Lx_edge_buttonGroup.setObjectName(_fromUtf8("Lx_edge_buttonGroup"))
        self.Lx_edge_buttonGroup.addButton(self.AC_edge_radioButton)
        self.verticalLayout_3.addWidget(self.AC_edge_radioButton)
        self.ZZ_edge_radioButton = QtGui.QRadioButton(self.graphene_generator_tabWidget)
        self.ZZ_edge_radioButton.setObjectName(_fromUtf8("ZZ_edge_radioButton"))
        self.Lx_edge_buttonGroup.addButton(self.ZZ_edge_radioButton)
        self.verticalLayout_3.addWidget(self.ZZ_edge_radioButton)
        self.horizontalLayout_20.addLayout(self.verticalLayout_3)
        self.verticalLayout_7.addLayout(self.horizontalLayout_20)
        self.line_4 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_4.setFrameShape(QtGui.QFrame.HLine)
        self.line_4.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_4.setObjectName(_fromUtf8("line_4"))
        self.verticalLayout_7.addWidget(self.line_4)
        self.horizontalLayout_32 = QtGui.QHBoxLayout()
        self.horizontalLayout_32.setObjectName(_fromUtf8("horizontalLayout_32"))
        self.horizontalLayout_33 = QtGui.QHBoxLayout()
        self.horizontalLayout_33.setObjectName(_fromUtf8("horizontalLayout_33"))
        self.horizontalLayout_34 = QtGui.QHBoxLayout()
        self.horizontalLayout_34.setObjectName(_fromUtf8("horizontalLayout_34"))
        self.label_19 = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(18)
        self.label_19.setFont(font)
        self.label_19.setObjectName(_fromUtf8("label_19"))
        self.horizontalLayout_34.addWidget(self.label_19)
        self.graphene_element1_comboBox = QtGui.QComboBox(self.graphene_generator_tabWidget)
        self.graphene_element1_comboBox.setObjectName(_fromUtf8("graphene_element1_comboBox"))
        self.graphene_element1_comboBox.addItem(_fromUtf8(""))
        self.graphene_element1_comboBox.addItem(_fromUtf8(""))
        self.graphene_element1_comboBox.addItem(_fromUtf8(""))
        self.graphene_element1_comboBox.addItem(_fromUtf8(""))
        self.graphene_element1_comboBox.addItem(_fromUtf8(""))
        self.horizontalLayout_34.addWidget(self.graphene_element1_comboBox)
        self.horizontalLayout_33.addLayout(self.horizontalLayout_34)
        self.horizontalLayout_35 = QtGui.QHBoxLayout()
        self.horizontalLayout_35.setObjectName(_fromUtf8("horizontalLayout_35"))
        self.label_20 = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(18)
        self.label_20.setFont(font)
        self.label_20.setObjectName(_fromUtf8("label_20"))
        self.horizontalLayout_35.addWidget(self.label_20)
        self.graphene_element2_comboBox = QtGui.QComboBox(self.graphene_generator_tabWidget)
        self.graphene_element2_comboBox.setObjectName(_fromUtf8("graphene_element2_comboBox"))
        self.graphene_element2_comboBox.addItem(_fromUtf8(""))
        self.graphene_element2_comboBox.addItem(_fromUtf8(""))
        self.graphene_element2_comboBox.addItem(_fromUtf8(""))
        self.graphene_element2_comboBox.addItem(_fromUtf8(""))
        self.graphene_element2_comboBox.addItem(_fromUtf8(""))
        self.horizontalLayout_35.addWidget(self.graphene_element2_comboBox)
        self.horizontalLayout_33.addLayout(self.horizontalLayout_35)
        self.horizontalLayout_32.addLayout(self.horizontalLayout_33)
        spacerItem8 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_32.addItem(spacerItem8)
        self.verticalLayout_7.addLayout(self.horizontalLayout_32)
        self.horizontalLayout_27 = QtGui.QHBoxLayout()
        self.horizontalLayout_27.setObjectName(_fromUtf8("horizontalLayout_27"))
        self.horizontalLayout_36 = QtGui.QHBoxLayout()
        self.horizontalLayout_36.setObjectName(_fromUtf8("horizontalLayout_36"))
        self.graphene_e1e2_bond_label = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.graphene_e1e2_bond_label.setFont(font)
        self.graphene_e1e2_bond_label.setObjectName(_fromUtf8("graphene_e1e2_bond_label"))
        self.horizontalLayout_36.addWidget(self.graphene_e1e2_bond_label)
        self.graphene_bond_doubleSpinBox = QtGui.QDoubleSpinBox(self.graphene_generator_tabWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.graphene_bond_doubleSpinBox.sizePolicy().hasHeightForWidth())
        self.graphene_bond_doubleSpinBox.setSizePolicy(sizePolicy)
        self.graphene_bond_doubleSpinBox.setMinimumSize(QtCore.QSize(150, 0))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.graphene_bond_doubleSpinBox.setFont(font)
        self.graphene_bond_doubleSpinBox.setInputMethodHints(QtCore.Qt.ImhFormattedNumbersOnly)
        self.graphene_bond_doubleSpinBox.setReadOnly(False)
        self.graphene_bond_doubleSpinBox.setDecimals(3)
        self.graphene_bond_doubleSpinBox.setMinimum(0.5)
        self.graphene_bond_doubleSpinBox.setMaximum(10.0)
        self.graphene_bond_doubleSpinBox.setSingleStep(0.01)
        self.graphene_bond_doubleSpinBox.setProperty("value", 1.42)
        self.graphene_bond_doubleSpinBox.setObjectName(_fromUtf8("graphene_bond_doubleSpinBox"))
        self.horizontalLayout_36.addWidget(self.graphene_bond_doubleSpinBox)
        self.horizontalLayout_27.addLayout(self.horizontalLayout_36)
        spacerItem9 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_27.addItem(spacerItem9)
        self.verticalLayout_7.addLayout(self.horizontalLayout_27)
        self.line_7 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_7.setFrameShape(QtGui.QFrame.HLine)
        self.line_7.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_7.setObjectName(_fromUtf8("line_7"))
        self.verticalLayout_7.addWidget(self.line_7)
        self.horizontalLayout_22 = QtGui.QHBoxLayout()
        self.horizontalLayout_22.setObjectName(_fromUtf8("horizontalLayout_22"))
        self.horizontalLayout_21 = QtGui.QHBoxLayout()
        self.horizontalLayout_21.setObjectName(_fromUtf8("horizontalLayout_21"))
        self.horizontalLayout_18 = QtGui.QHBoxLayout()
        self.horizontalLayout_18.setObjectName(_fromUtf8("horizontalLayout_18"))
        self.label_11 = QtGui.QLabel(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label_11.setFont(font)
        self.label_11.setObjectName(_fromUtf8("label_11"))
        self.horizontalLayout_18.addWidget(self.label_11)
        self.nlayers_spinBox = QtGui.QSpinBox(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.nlayers_spinBox.setFont(font)
        self.nlayers_spinBox.setMinimum(1)
        self.nlayers_spinBox.setMaximum(50)
        self.nlayers_spinBox.setObjectName(_fromUtf8("nlayers_spinBox"))
        self.horizontalLayout_18.addWidget(self.nlayers_spinBox)
        self.horizontalLayout_21.addLayout(self.horizontalLayout_18)
        self.line_6 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_6.setFrameShape(QtGui.QFrame.VLine)
        self.line_6.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_6.setObjectName(_fromUtf8("line_6"))
        self.horizontalLayout_21.addWidget(self.line_6)
        self.horizontalLayout_22.addLayout(self.horizontalLayout_21)
        self.verticalLayout_4 = QtGui.QVBoxLayout()
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.label_6 = QtGui.QLabel(self.graphene_generator_tabWidget)
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.verticalLayout_4.addWidget(self.label_6)
        self.AA_stacking_radioButton = QtGui.QRadioButton(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.AA_stacking_radioButton.setFont(font)
        self.AA_stacking_radioButton.setCheckable(True)
        self.AA_stacking_radioButton.setChecked(True)
        self.AA_stacking_radioButton.setObjectName(_fromUtf8("AA_stacking_radioButton"))
        self.stacking_order_buttonGroup = QtGui.QButtonGroup(NanoGen)
        self.stacking_order_buttonGroup.setObjectName(_fromUtf8("stacking_order_buttonGroup"))
        self.stacking_order_buttonGroup.addButton(self.AA_stacking_radioButton)
        self.verticalLayout_4.addWidget(self.AA_stacking_radioButton)
        self.AB_stacking_radioButton = QtGui.QRadioButton(self.graphene_generator_tabWidget)
        font = QtGui.QFont()
        font.setPointSize(20)
        self.AB_stacking_radioButton.setFont(font)
        self.AB_stacking_radioButton.setCheckable(True)
        self.AB_stacking_radioButton.setObjectName(_fromUtf8("AB_stacking_radioButton"))
        self.stacking_order_buttonGroup.addButton(self.AB_stacking_radioButton)
        self.verticalLayout_4.addWidget(self.AB_stacking_radioButton)
        self.horizontalLayout_22.addLayout(self.verticalLayout_4)
        self.verticalLayout_7.addLayout(self.horizontalLayout_22)
        self.line_9 = QtGui.QFrame(self.graphene_generator_tabWidget)
        self.line_9.setFrameShape(QtGui.QFrame.HLine)
        self.line_9.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_9.setObjectName(_fromUtf8("line_9"))
        self.verticalLayout_7.addWidget(self.line_9)
        spacerItem10 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_7.addItem(spacerItem10)
        self.horizontalLayout_37.addLayout(self.verticalLayout_7)
        spacerItem11 = QtGui.QSpacerItem(92, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_37.addItem(spacerItem11)
        self.verticalLayout_8.addLayout(self.horizontalLayout_37)
        self.nanogen_tabWidget.addTab(self.graphene_generator_tabWidget, _fromUtf8(""))
        self.fullerene_generators = QtGui.QWidget()
        self.fullerene_generators.setObjectName(_fromUtf8("fullerene_generators"))
        self.verticalLayout_11 = QtGui.QVBoxLayout(self.fullerene_generators)
        self.verticalLayout_11.setObjectName(_fromUtf8("verticalLayout_11"))
        self.listWidget = QtGui.QListWidget(self.fullerene_generators)
        self.listWidget.setObjectName(_fromUtf8("listWidget"))
        self.verticalLayout_11.addWidget(self.listWidget)
        self.nanogen_tabWidget.addTab(self.fullerene_generators, _fromUtf8(""))
        self.verticalLayout_12.addWidget(self.nanogen_tabWidget)
        self.line_3 = QtGui.QFrame(self.centralwidget)
        self.line_3.setFrameShape(QtGui.QFrame.HLine)
        self.line_3.setFrameShadow(QtGui.QFrame.Sunken)
        self.line_3.setObjectName(_fromUtf8("line_3"))
        self.verticalLayout_12.addWidget(self.line_3)
        self.verticalLayout_5 = QtGui.QVBoxLayout()
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.horizontalLayout_25 = QtGui.QHBoxLayout()
        self.horizontalLayout_25.setObjectName(_fromUtf8("horizontalLayout_25"))
        self.label_16 = QtGui.QLabel(self.centralwidget)
        self.label_16.setObjectName(_fromUtf8("label_16"))
        self.horizontalLayout_25.addWidget(self.label_16)
        self.save_directory_lineEdit = QtGui.QLineEdit(self.centralwidget)
        self.save_directory_lineEdit.setObjectName(_fromUtf8("save_directory_lineEdit"))
        self.horizontalLayout_25.addWidget(self.save_directory_lineEdit)
        self.verticalLayout_5.addLayout(self.horizontalLayout_25)
        self.horizontalLayout_15 = QtGui.QHBoxLayout()
        self.horizontalLayout_15.setObjectName(_fromUtf8("horizontalLayout_15"))
        spacerItem12 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(spacerItem12)
        self.structure_format_comboBox = QtGui.QComboBox(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(18)
        self.structure_format_comboBox.setFont(font)
        self.structure_format_comboBox.setModelColumn(0)
        self.structure_format_comboBox.setObjectName(_fromUtf8("structure_format_comboBox"))
        self.structure_format_comboBox.addItem(_fromUtf8(""))
        self.structure_format_comboBox.addItem(_fromUtf8(""))
        self.horizontalLayout_15.addWidget(self.structure_format_comboBox)
        self.generate_pushButton = QtGui.QPushButton(self.centralwidget)
        font = QtGui.QFont()
        font.setFamily(_fromUtf8("Arial"))
        font.setPointSize(18)
        self.generate_pushButton.setFont(font)
        self.generate_pushButton.setObjectName(_fromUtf8("generate_pushButton"))
        self.horizontalLayout_15.addWidget(self.generate_pushButton)
        spacerItem13 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_15.addItem(spacerItem13)
        self.verticalLayout_5.addLayout(self.horizontalLayout_15)
        self.verticalLayout_12.addLayout(self.verticalLayout_5)
        NanoGen.setCentralWidget(self.centralwidget)
        self.nanogen_statusBar = QtGui.QStatusBar(NanoGen)
        self.nanogen_statusBar.setObjectName(_fromUtf8("nanogen_statusBar"))
        NanoGen.setStatusBar(self.nanogen_statusBar)

        self.retranslateUi(NanoGen)
        self.nanogen_tabWidget.setCurrentIndex(0)
        self.nanotube_element1_comboBox.setCurrentIndex(3)
        self.nanotube_element2_comboBox.setCurrentIndex(3)
        self.graphene_element1_comboBox.setCurrentIndex(3)
        self.graphene_element2_comboBox.setCurrentIndex(3)
        self.structure_format_comboBox.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(NanoGen)

    def retranslateUi(self, NanoGen):
        NanoGen.setWindowTitle(_translate("NanoGen", "NanoGen", None))
        self.nanotube_generator_radioButton.setText(_translate("NanoGen", "Nanotube Generator", None))
        self.capped_nanotube_generator_radioButton.setText(_translate("NanoGen", "Capped Nanotube Generator", None))
        self.mwnt_generator_radioButton.setText(_translate("NanoGen", "Multi-walled Nanotube Generator", None))
        self.unrolled_nanotube_generator_radioButton.setText(_translate("NanoGen", "Unrolled Nanotube Generator", None))
        self.bundle_generator_checkBox.setText(_translate("NanoGen", "Bundle Generator", None))
        self.label_7.setText(_translate("NanoGen", "<html><head/><body><p align=\"right\">n = </p></body></html>", None))
        self.label_14.setText(_translate("NanoGen", "<html><head/><body><p align=\"right\">m = </p></body></html>", None))
        self.label_17.setText(_translate("NanoGen", "Element 1:", None))
        self.nanotube_element1_comboBox.setItemText(0, _translate("NanoGen", "H", None))
        self.nanotube_element1_comboBox.setItemText(1, _translate("NanoGen", "He", None))
        self.nanotube_element1_comboBox.setItemText(2, _translate("NanoGen", "B", None))
        self.nanotube_element1_comboBox.setItemText(3, _translate("NanoGen", "C", None))
        self.nanotube_element1_comboBox.setItemText(4, _translate("NanoGen", "N", None))
        self.label_18.setText(_translate("NanoGen", "Element 2:", None))
        self.nanotube_element2_comboBox.setItemText(0, _translate("NanoGen", "H", None))
        self.nanotube_element2_comboBox.setItemText(1, _translate("NanoGen", "He", None))
        self.nanotube_element2_comboBox.setItemText(2, _translate("NanoGen", "B", None))
        self.nanotube_element2_comboBox.setItemText(3, _translate("NanoGen", "C", None))
        self.nanotube_element2_comboBox.setItemText(4, _translate("NanoGen", "N", None))
        self.nanotube_e1e2_bond_label.setText(_translate("NanoGen", "<html><head/><body><p>C-C bond =</p></body></html>", None))
        self.nanotube_bond_doubleSpinBox.setSuffix(_translate("NanoGen", " Å", None))
        self.label.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">n</span><span style=\" vertical-align:sub;\">x </span>=</p></body></html>", None))
        self.label_2.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">x </span>= </p></body></html>", None))
        self.label_3.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">n</span><span style=\" vertical-align:sub;\">y </span>=</p></body></html>", None))
        self.label_12.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">y </span>= </p></body></html>", None))
        self.label_4.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">n</span><span style=\" vertical-align:sub;\">z </span>=</p></body></html>", None))
        self.label_13.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">z </span>= </p></body></html>", None))
        self.Lz_doubleSpinBox.setSuffix(_translate("NanoGen", " nm", None))
        self.nanogen_tabWidget.setTabText(self.nanogen_tabWidget.indexOf(self.nanotube_generator_tabWidget), _translate("NanoGen", "Nanotubes", None))
        self.label_9.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">x</span> =</p></body></html>", None))
        self.length_doubleSpinBox.setSuffix(_translate("NanoGen", " nm", None))
        self.label_10.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">y</span> =</p></body></html>", None))
        self.width_doubleSpinBox.setSuffix(_translate("NanoGen", " nm", None))
        self.label_5.setText(_translate("NanoGen", "<html><head/><body><p align=\"center\"><span style=\" font-style:italic;\">L</span><span style=\" vertical-align:sub;\">x</span> edge</p></body></html>", None))
        self.AC_edge_radioButton.setText(_translate("NanoGen", "armchair", None))
        self.ZZ_edge_radioButton.setText(_translate("NanoGen", "zigzag", None))
        self.label_19.setText(_translate("NanoGen", "Element 1:", None))
        self.graphene_element1_comboBox.setItemText(0, _translate("NanoGen", "H", None))
        self.graphene_element1_comboBox.setItemText(1, _translate("NanoGen", "He", None))
        self.graphene_element1_comboBox.setItemText(2, _translate("NanoGen", "B", None))
        self.graphene_element1_comboBox.setItemText(3, _translate("NanoGen", "C", None))
        self.graphene_element1_comboBox.setItemText(4, _translate("NanoGen", "N", None))
        self.label_20.setText(_translate("NanoGen", "Element 2:", None))
        self.graphene_element2_comboBox.setItemText(0, _translate("NanoGen", "H", None))
        self.graphene_element2_comboBox.setItemText(1, _translate("NanoGen", "He", None))
        self.graphene_element2_comboBox.setItemText(2, _translate("NanoGen", "B", None))
        self.graphene_element2_comboBox.setItemText(3, _translate("NanoGen", "C", None))
        self.graphene_element2_comboBox.setItemText(4, _translate("NanoGen", "N", None))
        self.graphene_e1e2_bond_label.setText(_translate("NanoGen", "<html><head/><body><p>C-C bond =</p></body></html>", None))
        self.graphene_bond_doubleSpinBox.setSuffix(_translate("NanoGen", " Å", None))
        self.label_11.setText(_translate("NanoGen", "<html><head/><body><p><span style=\" font-style:italic;\">n</span><span style=\" vertical-align:sub;\">layers</span> =</p></body></html>", None))
        self.label_6.setText(_translate("NanoGen", "<html><head/><body><p>Stacking Order</p></body></html>", None))
        self.AA_stacking_radioButton.setText(_translate("NanoGen", "AA", None))
        self.AB_stacking_radioButton.setText(_translate("NanoGen", "AB", None))
        self.nanogen_tabWidget.setTabText(self.nanogen_tabWidget.indexOf(self.graphene_generator_tabWidget), _translate("NanoGen", "Graphene", None))
        self.nanogen_tabWidget.setTabText(self.nanogen_tabWidget.indexOf(self.fullerene_generators), _translate("NanoGen", "Fullerenes", None))
        self.label_16.setText(_translate("NanoGen", "Save directory:", None))
        self.structure_format_comboBox.setItemText(0, _translate("NanoGen", "LAMMPS data", None))
        self.structure_format_comboBox.setItemText(1, _translate("NanoGen", "xyz", None))
        self.generate_pushButton.setText(_translate("NanoGen", "Generate", None))




# -*- coding: utf-8 -*-
"""
=============================================================================
Nanotube defect generator (:mod:`sknano.nanogen._nanotube_defect_generators`)
=============================================================================

.. currentmodule:: sknano.nanogen._nanotube_defect_generators

"""
from __future__ import absolute_import, division, print_function
__docformat__ = 'restructuredtext en'

import itertools

import numpy as np

from ..tools.refdata import CCbond

from ._defect_generators import DefectGenerator, CrossLinkedDefectGenerator, \
    StoneWalesDefectGenerator, VacancyGenerator
from ._nanotube_bundle_generators import NanotubeBundleGenerator

__all__ = ['NanotubeDefectGenerator',
           'NanotubeVacancyGenerator',
           'CrossLinkedNanotubeBundleGenerator']


class NanotubeDefectGenerator(DefectGenerator, StoneWalesDefectGenerator):
    pass


class CrossLinkedNanotubeBundleGenerator(CrossLinkedDefectGenerator):
    pass


class NanotubeVacancyGenerator(VacancyGenerator):
    """Generate vacancies in nanotubes.

    .. versionadded:: 0.2.6

    Parameters
    ----------
    n, m : int, optional
        Chiral indices defining the nanotube chiral vector
        :math:`\\mathbf{C}_{h} = n\\mathbf{a}_{1} + m\\mathbf{a}_{2} = (n, m)`.
    nx, ny, nz : int, optional
        Number of repeat unit cells in the :math:`x, y, z` dimensions.
    fname : str, optional
        Structure data filename. If you don't provide a structure data file,
        the you **must** provide the structure data parameters to
        generate the structure data file.
    structure_format : {None, str}, optional
        Chemical file format of saved structure data.
        If `None`, then guess based on `fname` file extension.
        Otherwise, must be one of:

            - xyz
            - data

    element1, element2 : {str, int}, optional
        Element symbol or atomic number of basis atoms 1 and 2
    bond : float, optional
        :math:`\\mathrm{a}_{\\mathrm{CC}} =` distance between
        nearest neighbor atoms. Must be in units of **Angstroms**.
    vdw_spacing : float, optional
        van der Waals distance between nearest neighbor tubes
    bundle_packing : {None, 'hexagonal', 'cubic'}, optional
        close packing arrangement of bundles
    bundle_geometry : {None, 'triangle', 'hexagon', 'square', 'rectangle',
                       'rhombus', 'rhomboid'}, optional
    Lx, Ly, Lz : float, optional
        length of bundle in :math:`x, y, z` dimensions in **nanometers**.
        Overrides the :math:`n_x, n_y, n_z` cell values.
    verbose : bool, optional
        Verbose output

    Raises
    ------
    `TypeError`
        If `fname` is `None` and `n` and `m` are not integers.

    Examples
    --------

    The `NanotubeVacancyGenerator` class behaves nearly
    identically to the `GrapheneVacancyGenerator` class.
    You can provide any existing structure data file in any supported
    format or else provide structure parameters to generate a structure
    in real-time. The `NanotubeVacancyGenerator` uses
    the `NanotubeBundleGenerator` class to generate
    nanotubes. See the structure generator classes provided by the
    :mod:`~sknano.nanogen` module for more detailed docs.

    In the next example, we'll generate a nanotube bundle and then
    poke some holes in it.

    >>> from sknano.nanogen import NanotubeVacancyGenerator
    >>> ntvg = NanotubeVacancyGenerator(n=10, m=5, Lz=10, fix_Lz=True,
    ...                                 bundle_geometry='hexagon')

    Notice that I used the `Lz` keyword argument to specify the
    **length** of the nanotube bundle in **nanometers**, and I also
    set `fix_Lz=True`. If you don't set `fix_Lz=True`, then the
    length of the nanotube will get truncated such that it's an integer
    multiple of the unit cell length, which,
    depending on the chirality, may end up being very different than
    the desired length set by `Lz`.

    Next, I add 35 **double** vacancies distributed uniformly along the
    :math:`z`-axis. For the `NanotubeVacancyGenerator`, if `uniform` is `True`,
    the code is written such it will divide the total number of vacancies
    evenly among all nanotubes and distribute those vacancies uniformly along
    the specified `bin_axis`. The circumferential location of vacancies is
    selected randomly.  Future revisions will allow more precise control of the
    positioning.

    >>> ntvg.generate_vacancy_structure(Nvac_sites=35, uniform=True,
    ...                                 bin_axis='z', vac_type='double')

    Running that command left me with the saved vacancy structure data which
    I loaded into VMD. I used the VMD selection command that was printed to the
    terminal window to highlight the atoms and bonds surrounding the vacancies.

    The rendered structure looks like:

    .. image:: /images/1005_hcp_7tube_hexagon+70_vacancies-01.png

    Here's a final example which generates a *tri-vacancy* at each
    `Nvac_sites`.

    >>> ntvg = NanotubeVacancyGenerator(n=10, m=10, nz=50)
    >>> ntvg.generate_vacancy_structure(Nvac_sites=20, uniform=True,
    ...                                 bin_axis='z', vac_type='triple')

    After relaxing the vacancy structure data in LAMMPS, the rendered
    structure looks like:

    .. image:: /images/1010_hcp_1cellx1cellx50cells+60_vacancies-01.png

    This poor nanotube has taken quite a beating.

    """
    def __init__(self, n=None, m=None, nx=1, ny=1, nz=1,
                 element1='C', element2='C', bond=CCbond, vdw_spacing=3.4,
                 bundle_packing=None, bundle_geometry=None,
                 Lx=None, Ly=None, Lz=None, fix_Lz=False,
                 rotate_structure=False, rotation_angle=None,
                 rotation_axis=None,
                 fname=None, outpath=None, structure_format=None,
                 verbose=False):

        self._Ntubes = None
        self._Natoms_per_tube = None

        if fname is None and isinstance(n, int) and isinstance(m, int):
            ntbg = NanotubeBundleGenerator(n=n, m=m, nx=nx, ny=ny, nz=nz,
                                           element1=element1,
                                           element2=element2,
                                           bond=bond,
                                           vdw_spacing=vdw_spacing,
                                           bundle_packing=bundle_packing,
                                           bundle_geometry=bundle_geometry,
                                           Lx=Lx, Ly=Ly, Lz=Lz,
                                           fix_Lz=fix_Lz,
                                           verbose=verbose)
            ntbg.save_data(structure_format='data')
            fname = ntbg.fname
            structure_format = ntbg.structure_format
            self._Ntubes = ntbg.Ntubes
            self._Natoms_per_tube = ntbg.Natoms_per_tube
        else:
            raise TypeError('Either `fname` but be provided or '
                            '`n` and `m` must be specified as integers')

        super(NanotubeVacancyGenerator, self).__init__(
            fname=fname, outpath=outpath, structure_format=structure_format,
            verbose=verbose)

    def generate_vacancy_structure(self, Nvac_sites=None, Nvac_clusters=None,
                                   cluster_size=None, vac_type='single',
                                   uniform=False, bin_axis='z', Ntubes=None,
                                   vmd_selection_radius=np.sqrt(10.5),
                                   show_vmd_selection_cmd=True):
        """Generate vacancy structure.

        Parameters
        ----------
        Nvac_sites : int, optional
            total number of vacancy sites to "add" to structure data.
        Nvac_clusters : int, optional
            total number of vacancy cluster sites to "add" to structure data.
        random : bool, optional
            Generate random vacancies in structure data.
        uniform : bool, optional
            Generate uniform vacancies in structure data.
        bin_axis : {'x', 'y', 'z'}, optional
            axis along which to generate uniform vacancies
        Ntubes : {None, int}, optional
        show_vmd_selection_cmd : bool, optional
            Generate a VMD selection string that can be used to
            select the atoms surrounding the vacancies.
        vmd_selection_radius : float, optional
            Cutoff radius for VMD selection command in units of **Angstroms**.

        Raises
        ------
        TypeError
            If `Nvac_sites` is `None` and `Nvac_clusters` is `None` or
            if `uniform` is `True` and the `structure_data` was read in
            from a file and `Ntubes` is `None`.

        """

        if Nvac_sites is None and Nvac_clusters is None:
            raise TypeError('`Nvac_sites` or `Nvac_clusters` must be an '
                            'integer.')
        elif Nvac_sites is None and Nvac_clusters is not None:
            Nvac_sites = Nvac_clusters
            if cluster_size is None:
                cluster_size = 1

        self._Nvac_sites = Nvac_sites
        self._Nvac_clusters = Nvac_clusters
        self._cluster_size = cluster_size
        self._vac_type = vac_type
        self._vmd_selection_radius = vmd_selection_radius
        self._show_vmd_selection_cmd = show_vmd_selection_cmd

        if uniform:
            if Ntubes is None:
                Ntubes = self._Ntubes
            if not isinstance(Ntubes, int):
                raise TypeError('`Ntubes` must be specified as an integer.')

            Natoms = len(self._atom_ids)
            if self._Natoms_per_tube is not None:
                Natoms_per_tube = self._Natoms_per_tube
            else:
                Natoms_per_tube = int(Natoms / Ntubes)

            Nvac_sites_per_tube = int(Nvac_sites / Ntubes)
            extra = Nvac_sites % Ntubes
            nbins = Ntubes * Nvac_sites_per_tube

            bin_sets = []
            for n in xrange(Ntubes):
                bin_sets.append(np.arange(n, Nvac_sites, Ntubes))
            bin_set_iter = itertools.cycle((bin_sets))

            vac_bin_edges = np.linspace(self._atom_coords[bin_axis].min(),
                                        self._atom_coords[bin_axis].max(),
                                        num=nbins+1)
            vac_coords_along_bin_axis = \
                vac_bin_edges[:-1] + np.diff(vac_bin_edges) / 2

            if self._verbose:
                print('Ntubes: {}'.format(Ntubes))
                print('Natoms: {}'.format(Natoms))
                print('Natoms_per_tube: {}'.format(Natoms_per_tube))
                print('Nvac_sites: {}'.format(Nvac_sites))
                print('Nvac_sites_per_tube: {}'.format(Nvac_sites_per_tube))
                print('extra vacancies: {}'.format(extra))
                print('nbins: {}'.format(nbins))
                print('bin_sets:\n{}'.format(bin_sets))
                print('vac_bin_edges:'
                      '\n{}'.format(vac_bin_edges))
                print('vac_coords_along_bin_axis:'
                      '\n{}'.format(vac_coords_along_bin_axis))

            for i in xrange(Ntubes):
                tube_atom_indices = \
                    np.where((self._atom_ids > (Natoms_per_tube * i)) &
                             (self._atom_ids <= (Natoms_per_tube * (i + 1))))

                tube_atom_ids = self._atom_ids[tube_atom_indices]

                tube_coords = self._atoms.get_filtered_coords(tube_atom_ids,
                                                              as_dict=True,
                                                              invert=False)

                if self._verbose:
                    print('tube n: {:d}'.format(i + 1))
                    print('tube_atom_ids:\n{}'.format(tube_atom_ids))
                    print('tube_atom_coords:\n{}'.format(tube_coords))

                bin_set = bin_set_iter.next()
                for vac_pos in vac_coords_along_bin_axis[bin_set]:
                    candidate_vac_atom_indices = \
                        np.where(np.abs(tube_coords[bin_axis] - vac_pos) <= 1)
                    candidate_vac_atom_ids = \
                        tube_atom_ids[candidate_vac_atom_indices]

                    if self._verbose:
                        print(u'vac_pos along {}-axis: {:.2f} \u00c5'.format(
                            bin_axis, vac_pos))
                        print('N candidate_vac_atom_ids: '
                              '{}\n'.format(len(candidate_vac_atom_ids)))

                    self._vac_ids = \
                        np.r_[self._vac_ids,
                              np.random.choice(candidate_vac_atom_ids)]
        else:
            super(NanotubeVacancyGenerator, self)._random_vacancy_generator()

        super(NanotubeVacancyGenerator, self)._generate_vacancy_structure()



# -*- coding: utf-8 -*-
"""
=============================================================================
Graphene defect generator (:mod:`sknano.nanogen._graphene_defect_generators`)
=============================================================================

.. currentmodule:: sknano.nanogen._graphene_defect_generators

"""
from __future__ import absolute_import, division, print_function
__docformat__ = 'restructuredtext en'

import itertools

import numpy as np

from ..tools.refdata import CCbond

from ._defect_generators import DefectGenerator, VacancyGenerator
from ._graphene_generators import GrapheneGenerator
from ._nanotube_generators import UnrolledNanotubeGenerator

__all__ = ['GrapheneDefectGenerator', 'GrapheneVacancyGenerator']


class GrapheneDefectGenerator(DefectGenerator):
    pass


class GrapheneVacancyGenerator(VacancyGenerator):
    """Generate vacancies in graphene.

    .. versionadded:: 0.2.6

    Parameters
    ----------
    length : float, optional
        Length of graphene sheet in **nanometers**
    width : float, optional
        Width of graphene sheet in **nanometers**
    edge : {'AC', 'armchair', 'ZZ', 'zigzag'}, optional
        **A**\ rm\ **C**\ hair or **Z**\ ig\ **Z**\ ag edge along
        the `length` of the sheet.
    fname : str, optional
        Structure data filename. If you don't provide a structure data file,
        the you **must** provide the structure data parameters to
        generate the structure data file.
    outpath : str, optional
        Output path for structure data file.
    structure_format : {None, str}, optional
        Chemical file format of saved structure data.
        If `None`, then guess based on `fname` file extension.
        Otherwise, must be one of:

            - `xyz`
            - `data`

    element1, element2 : {str, int}, optional
        Element symbol or atomic number of basis atoms 1 and 2.
    bond : float, optional
        bond length between nearest-neighbor atoms in **Angstroms**.
    nlayers : int, optional
        Number of graphene layers.
    layer_spacing : float, optional
        Distance between layers in **Angstroms**.
    stacking_order : {'AA', 'AB'}, optional
        Stacking order of graphene layers
    verbose : bool, optional
        Verbose output

    Raises
    ------
    `TypeError`
        If `fname` is `None` and `n` and `m` are not integers or
        `length` and `width` are not floats.

    Examples
    --------

    Import the `GrapheneVacancyGenerator` class.

    >>> from sknano.nanogen import GrapheneVacancyGenerator

    You can supply any existing structure data file
    (as long as its a supported format) to
    `GrapheneVacancyGenerator` using the
    `fname` keyword argument. For example, if you have an existing
    LAMMPS structure data file for graphene named
    *10nmx5nm_ZZ_1layer.data*, you could load that into your
    vacancy generator by typing:

    >>> gvacgen = GrapheneVacancyGenerator(fname='10nmx5nm_ZZ_1layer.data')

    If say, you had your LAMMPS data file saved with a `.lammps`
    extension, then you would need to specify the `structure_format`
    keyword argument as well:

    >>> gvacgen = GrapheneVacancyGenerator(fname='10nmx5nm_ZZ_1layer.lammps',
    ...                                    structure_format='data')

    If you don't have an existing structure data file and want to generate
    your graphene structure on-the-fly, then you need to provide
    structure parameters to the `GrapheneVacancyGenerator`
    constructor. These arguments are passed to an instance of
    `GrapheneGenerator` "under the hood" (see the `GrapheneGenerator` docs
    for examples and more detailed documentation). The `length` and `width`
    parameters are required.

    In the following example, we'll use the `GrapheneVacancyGenerator` class
    to generate a *10nm x 2nm*, single layer graphene with `armchair` edges,
    and then poke some holes in it.

    >>> gvacgen = GrapheneVacancyGenerator(length=10, width=2, edge='AC')

    Now let's add 20 *random* vacancies to the graphene layer by
    calling the `~GrapheneVacancyGenerator.generate_vacancy_structure`
    method:

    .. code-block:: python

       >>> gvacgen.generate_vacancy_structure(Nvac_sites=20)
       copy and paste the following VMD command to select
       the atoms surrounding the vacancies:

       (((x - -38.7646)^2 + (y - 0.0000)^2 + (z - 2.8420)^2) <= 12.00) or
       (((x - -38.7646)^2 + (y - 0.0000)^2 + (z - -5.6840)^2) <= 12.00) or
       (((x - -36.3034)^2 + (y - 0.0000)^2 + (z - -2.8420)^2) <= 12.00) or
       (((x - -36.3034)^2 + (y - 0.0000)^2 + (z - -7.1050)^2) <= 12.00) or
       (((x - -10.4603)^2 + (y - 0.0000)^2 + (z - 7.8155)^2) <= 12.00) or
       (((x - -6.7684)^2 + (y - 0.0000)^2 + (z - -2.8420)^2) <= 12.00) or
       (((x - -4.3072)^2 + (y - 0.0000)^2 + (z - -7.1050)^2) <= 12.00) or
       (((x - -3.0766)^2 + (y - 0.0000)^2 + (z - -3.5525)^2) <= 12.00) or
       (((x - 6.7684)^2 + (y - 0.0000)^2 + (z - -4.9735)^2) <= 12.00) or
       (((x - 6.7684)^2 + (y - 0.0000)^2 + (z - -9.2365)^2) <= 12.00) or
       (((x - 19.0746)^2 + (y - 0.0000)^2 + (z - 4.9735)^2) <= 12.00) or
       (((x - 22.7665)^2 + (y - 0.0000)^2 + (z - -9.9470)^2) <= 12.00) or
       (((x - 25.2278)^2 + (y - 0.0000)^2 + (z - -1.4210)^2) <= 12.00) or
       (((x - 27.6890)^2 + (y - 0.0000)^2 + (z - -7.1050)^2) <= 12.00) or
       (((x - 32.6115)^2 + (y - 0.0000)^2 + (z - 5.6840)^2) <= 12.00) or
       (((x - 33.8421)^2 + (y - 0.0000)^2 + (z - 0.7105)^2) <= 12.00) or
       (((x - 35.0727)^2 + (y - 0.0000)^2 + (z - -1.4210)^2) <= 12.00) or
       (((x - 37.5340)^2 + (y - 0.0000)^2 + (z - 5.6840)^2) <= 12.00) or
       (((x - 38.7646)^2 + (y - 0.0000)^2 + (z - -9.2365)^2) <= 12.00) or
       (((x - 39.9952)^2 + (y - 0.0000)^2 + (z - -9.9470)^2) <= 12.00)

    This will save the vacancy structure data in both
    `LAMMPS data` and `xyz` formats. You can load the `xyz` file
    into VMD and then copy the VMD selection command from the output above
    and paste it into VMD to highlight the atoms surrounding the vacancies.

    As you know, removing atoms from a material will change the equilibrium
    atomic arrangement of the atoms. Obtaining the new equilibrium structure
    in the vacancy structure data requires running molecular dynamics
    to compute the new forces on the atoms and move them to a minimum energy
    configuration. Currently this is not supported within this toolkit and
    requires using an external molecular dynamics package.  This toolkit
    outputs the structure data in the native `LAMMPS data` file format and is
    the molecular dynamics engine I used to relax the structure data after
    adding the vacancies.

    The rendered, relaxed structure, with vacancies highlighted, looks like:

    .. image:: /images/10nmx2nm_1layer_AC_CC_graphene+20_vacancies-01.png

    In addition adding single-vacancies, you can generate di-vacancies
    and tri-vacancies by setting `vac_type` to `double` and `triple`,
    respectively.

    Here's an example of generating single layer graphene with a
    `di-vacancy` at each of the 20 `Nvac_sites`.

    >>> gvacgen = GrapheneVacancyGenerator(length=10, width=5, edge='ZZ')
    >>> gvacgen.generate_vacancy_structure(Nvac_sites=20, vac_type='double')

    I'm not going to show the VMD selection command since it will not
    be the same if you run this example on your computer. After generating
    the vacancy structure, I relaxed the structure data using LAMMPS.

    Here's the rendered structure, with the 40 vacancies highlighted:

    .. image:: /images/10nmx5nm_1layer_ZZ_CC_graphene+40_vacancies-01.png

    """
    def __init__(self, n=None, m=None, nx=1, ny=1, nz=1,
                 length=None, width=None, edge=None,
                 element1='C', element2='C', bond=CCbond,
                 nlayers=1, layer_spacing=3.35, stacking_order='AB',
                 fname=None, outpath=None, structure_format=None,
                 rotate_structure=False, rotation_angle=None,
                 rotation_axis=None, verbose=False):

        if fname is None:
            if isinstance(length, (int, float)) and \
                    isinstance(width, (int, float)):
                gg = GrapheneGenerator(
                    length=length, width=width, edge=edge, element1=element1,
                    element2=element2, bond=bond, nlayers=nlayers,
                    layer_spacing=layer_spacing, stacking_order=stacking_order,
                    verbose=verbose)
            elif isinstance(n, int) and isinstance(m, int):
                gg = UnrolledNanotubeGenerator(
                    n=n, m=m, nx=nx, ny=ny, nz=nz,
                    element1=element1, element2=element2, bond=bond,
                    nlayers=nlayers, layer_spacing=layer_spacing,
                    stacking_order=stacking_order, verbose=verbose)
            else:
                raise TypeError('Either `fname` must be set or `n` and `m` '
                                'must be specified as integers or `length` '
                                'and `width` must be specified as floats.')
            gg.save_data(outpath=outpath, structure_format='data')
            fname = gg.fname
            structure_format = gg.structure_format

        super(GrapheneVacancyGenerator, self).__init__(
            fname=fname, outpath=outpath,
            structure_format=structure_format, verbose=verbose)

    def generate_vacancy_structure(self, Nvac_sites=None, Nvac_clusters=None,
                                   cluster_size=None, vac_type='single',
                                   uniform=False, bin_axis='z',
                                   distribute_evenly=False,
                                   vmd_selection_radius=np.sqrt(10.5),
                                   show_vmd_selection_cmd=True):
        """Generate vacancy structure.

        Parameters
        ----------
        Nvac_sites : int, optional
            total number of vacancy sites to "add" to structure data.
        Nvac_clusters : int, optional
            total number of vacancy cluster sites to "add" to structure data.
        uniform : bool, optional
            Generate vacancies uniformly distributed along `bin_axis`.
        show_vmd_selection_cmd : bool, optional
            Generate a VMD selection string that can be used to
            select the atoms surrounding the vacancies.
        vmd_selection_radius : float, optional
            Cutoff radius for VMD selection command in units of **Angstroms**.

        Raises
        ------
        `TypeError`
            If `Nvac_sites` is `None` and `Nvac_clusters` is `None`

        """
        if Nvac_sites is None and Nvac_clusters is None:
            raise TypeError('`Nvac_sites` or `Nvac_clusters` must '
                            'be an integer.')
        elif Nvac_sites is None and Nvac_clusters is not None:
            Nvac_sites = Nvac_clusters
            if cluster_size is None:
                cluster_size = 1

        self._Nvac_sites = Nvac_sites
        self._Nvac_clusters = Nvac_clusters
        self._cluster_size = cluster_size

        self._vac_type = vac_type
        self._vmd_selection_radius = vmd_selection_radius
        self._show_vmd_selection_cmd = show_vmd_selection_cmd

        if uniform:
            # find the coords of each layer
            y_coords = self._atom_coords['y']
            Nlayers = len(set(y_coords))
            layer_coords = np.asarray(sorted(list(set(y_coords))))

            Nvac_sites_per_layer = int(Nvac_sites / Nlayers)
            extra = Nvac_sites % Nlayers

            nbins = Nvac_sites_per_layer * Nlayers

            bin_sets = []
            for n in xrange(Nlayers):
                bin_sets.append(np.arange(n, nbins, Nlayers))
            bin_set_iter = itertools.cycle((bin_sets))

            bin_edges = np.linspace(self._atom_coords[bin_axis].min(),
                                    self._atom_coords[bin_axis].max(),
                                    num=nbins+1)

            bin_mid_pts = bin_edges[:-1] + np.diff(bin_edges) / 2

            if self._verbose:
                print('Nlayers: {}'.format(Nlayers))
                print('Nvac_sites: {}'.format(Nvac_sites))
                print('Nvac_sites_per_layer: {}'.format(Nvac_sites_per_layer))
                print('extra vacancies: {}'.format(extra))
                print('nbins: {}'.format(nbins))
                print('bin_sets:\n{}'.format(bin_sets))
                print('bin_edges:\n{}'.format(bin_edges))
                print('bin_mid_pts:\n{}'.format(bin_mid_pts))

            ortho_axis = 'x' if bin_axis == 'z' else 'z'
            ortho_mid_pt = self._atom_coords[ortho_axis].min() + \
                (self._atom_coords[ortho_axis].max() -
                 self._atom_coords[ortho_axis].min()) / 2
            atoms_along_ortho_axis = {'+': [], '-': []}

            for y in layer_coords:
                bin_set = bin_set_iter.next()
                for vac_pos in bin_mid_pts[bin_set]:
                    candidate_vac_atom_indices = \
                        np.where(
                            (self._atom_coords[ortho_axis] >=
                             (self._atom_coords[ortho_axis].min() + 2.5)) &
                            (self._atom_coords[ortho_axis] <=
                             (self._atom_coords[ortho_axis].max() - 2.5)) &
                            (np.abs(self._atom_coords['y'] - y) <= 0.5) &
                            (np.abs(self._atom_coords[bin_axis] - vac_pos)
                                <= 1))
                    candidate_vac_atom_ids = \
                        self._atom_ids[candidate_vac_atom_indices]

                    if self._verbose:
                        print('candidate_vac_atom_ids: '
                              '{}\n'.format(candidate_vac_atom_ids))

                    rand_vac_atom_id = None
                    while True:
                        rand_vac_atom_id = \
                            np.random.choice(candidate_vac_atom_ids)
                        if distribute_evenly:
                            rand_vac_atom = \
                                self._atoms.get_atoms(asarray=True)[
                                    self._atom_ids == rand_vac_atom_id][0]
                            ortho_pos = getattr(rand_vac_atom, ortho_axis)
                            if ortho_pos >= ortho_mid_pt and \
                                    len(atoms_along_ortho_axis['+']) < \
                                    Nvac_sites_per_layer / 2:
                                atoms_along_ortho_axis['+'].append(
                                    rand_vac_atom)
                                break
                            elif ortho_pos < ortho_mid_pt and \
                                    len(atoms_along_ortho_axis['-']) < \
                                    Nvac_sites_per_layer / 2:
                                atoms_along_ortho_axis['-'].append(
                                    rand_vac_atom)
                                break
                            else:
                                continue
                        else:
                            break

                    self._vac_ids = np.r_[self._vac_ids, rand_vac_atom_id]
        else:
            super(GrapheneVacancyGenerator, self)._random_vacancy_generator()

        super(GrapheneVacancyGenerator, self)._generate_vacancy_structure()



# -*- coding: utf-8 -*-
"""
========================================================================
Class for LAMMPS atoms (:mod:`sknano.structure_io.atoms._lammps_atoms`)
========================================================================

.. currentmodule:: sknano.structure_io.atoms._lammps_atoms

"""
from __future__ import absolute_import, division, print_function
__docformat__ = 'restructuredtext en'

from collections import OrderedDict

import numpy as np

try:
    from scipy.spatial import KDTree
    has_kdtree = True
except ImportError:
    print('Install scipy version >= 0.13.0 to allow '
          'nearest-neighbor queries between atoms.')
    has_kdtree = False

from ...tools import xyz_axes
from ._atoms import Atoms

__all__ = ['LAMMPSAtoms']


class LAMMPSAtoms(Atoms):
    """`Atoms` class for creating collection of `LAMMPSAtom` objects.

    Parameters
    ----------
    atoms : {None, sequence, `LAMMPSAtoms`}, optional
        if not `None`, then a list of `LAMMPSAtom` instance objects or an
        existing `LAMMPSAtoms` instance object.
    copylist : bool, optional
        perform shallow copy of atoms list
    deepcopy : bool, optional
        perform deepcopy of atoms list
    use_kdtree : bool, optional
        use :py:class:`~scipy:scipy.spatial.KDTree` to perform
        nearest-neighbor analysis.

    """

    def __init__(self, atoms=None, copylist=True, deepcopy=False,
                 use_kdtree=True):

        super(LAMMPSAtoms, self).__init__(
            atoms=atoms, copylist=copylist, deepcopy=deepcopy,
            update_property_lists=False)

        self._property_lists['q'] = self._charges = []
        self._property_lists['v'] = self._velocities = []
        self._property_lists['atomID'] = self._atom_ids = []
        self._property_lists['moleculeID'] = self._molecule_ids = []
        self._property_lists['CN'] = self._coordination_numbers = []
        self._property_lists['NN'] = self._nearest_neighbors = []
        #self._forces = []

        self._atomtypes = {}

        self._atom_tree = None

        if use_kdtree and has_kdtree is False:
            use_kdtree = False
        self._use_kdtree = use_kdtree

        self._NN_number = 6
        self._NN_cutoff = np.inf

        if atoms is not None:
            for atom in self._atoms:
                for p, plist in self._property_lists.iteritems():
                    plist.append(getattr(atom, p))

            if use_kdtree:
                self._atom_tree = self.atom_tree

    @property
    def atomtypes(self):
        """Return the atom type dictionary."""
        for atom in self._atoms:
            if atom.atomtype not in self._atomtypes:
                self._atomtypes[atom.atomtype] = {}
                self._atomtypes[atom.atomtype]['mass'] = atom.m
                self._atomtypes[atom.atomtype]['q'] = atom.q
        return self._atomtypes

    @property
    def atom_ids(self):
        """Return array of `LAMMPSAtom` IDs."""
        atom_ids = []
        for atom in self._atoms:
            atom_ids.append(atom.atomID)
        self._atom_ids = atom_ids[:]
        return np.asarray(self._atom_ids)

    @property
    def atom_tree(self):
        """Return the :py:class:`~scipy:scipy.spatial.KDTree` of coords."""
        if self._use_kdtree and len(self._atoms) != 0:
            self._atom_tree = KDTree(self.coords)
        return self._atom_tree

    @property
    def charges(self):
        """Return list of `LAMMPSAtom` charges."""
        #self._charges_array = np.asarray(self._charges)
        charges = []
        for atom in self._atoms:
            charges.append(atom.q)
        self._charges = charges[:]
        return np.asarray(self._charges)

    @property
    def coordination_numbers(self):
        """Return array of `LAMMPSAtom` coordination numbers."""
        self.update_coordination_numbers()
        coordination_numbers = []
        for atom in self._atoms:
            coordination_numbers.append(atom.CN)
        self._coordination_numbers = coordination_numbers[:]
        return np.asarray(self._coordination_numbers)

    def update_coordination_numbers(self):
        """Update `LAMMPSAtom` coordination numbers."""
        if self._use_kdtree:
            NN_d, NN_i = \
                self.query_atom_tree(n=self.NN_number,
                                     cutoff_radius=self.NN_cutoff)
            for i, atom in enumerate(self._atoms):
                CN = 0
                for d in NN_d[i]:
                    if d < self.NN_cutoff:
                        CN += 1
                atom.CN = CN

    @property
    def nearest_neighbors(self):
        """Return array of nearest-neighbor atoms for each `LAMMPSAtom`."""
        self.update_nearest_neighbors()
        nearest_neighbors = []
        for atom in self._atoms:
            nearest_neighbors.append(atom.NN)
        self._nearest_neighbors = nearest_neighbors[:]
        return np.asarray(self._nearest_neighbors)

    def update_nearest_neighbors(self):
        """Update `LAMMPSAtom` nearest-neighbors."""
        if self._use_kdtree:
            NN_d, NN_i = self.query_atom_tree(n=self.NN_number,
                                              cutoff_radius=self.NN_cutoff)
            for i, atom in enumerate(self._atoms):
                NN_atoms = []
                for j, d in enumerate(NN_d[i]):
                    if d < self.NN_cutoff:
                        NN_atoms.append(self._atoms[NN_i[i][j]])
                atom.NN = NN_atoms

    def query_atom_tree(self, n=6, eps=0, p=2, cutoff_radius=np.inf):
        """Query atom tree for nearest neighbors distances and indices.

        Parameters
        ----------
        n : integer
            The number of nearest neighbors to return.
        eps : nonnegative float
            Return approximate nearest neighbors; the kth returned value
            is guaranteed to be no further than (1+eps) times the
            distance to the real kth nearest neighbor.
        p : float, 1<=p<=infinity
            Which Minkowski p-norm to use.
            1 is the sum-of-absolute-values "Manhattan" distance
            2 is the usual Euclidean distance
            infinity is the maximum-coordinate-difference distance
        cutoff_radius : nonnegative float
            Return only neighbors within this distance. This is used to prune
            tree searches, so if you are doing a series of nearest-neighbor
            queries, it may help to supply the distance to the nearest neighbor
            of the most recent point.

        Returns
        -------
        NN_d : array of floats
            The distances to the nearest neighbors.
        NN_i : array of integers
            The locations of the neighbors in self.atom_tree.data. NN_i is the
            same shape as NN_d.

        """
        atom_tree = self.atom_tree
        NN_d = NN_i = None
        if atom_tree is not None:
            d, i = atom_tree.query(self.coords, k=n+1, eps=eps, p=p,
                                   distance_upper_bound=cutoff_radius)
            NN_d, NN_i = d[:, 1:], i[:, 1:]
        return NN_d, NN_i

    def query_coordination_numbers(self, n=6, rc=np.inf):
        """Query and update `LAMMPSAtom` coordination numbers.

        Parameters
        ----------
        n : int, optional
        rc : nonnegative float, optional

        """
        if self._use_kdtree:
            NN_d, NN_i = self.query_atom_tree(n=n, cutoff_radius=rc)
            for i, atom in enumerate(self._atoms):
                CN = 0
                for d in NN_d[i]:
                    if d < rc:
                        CN += 1
                atom.CN = CN
        coordination_numbers = []
        for atom in self._atoms:
            coordination_numbers.append(atom.CN)
        self._coordination_numbers = coordination_numbers[:]
        return np.asarray(self._coordination_numbers)

    def query_nearest_neighbors(self, n=6, rc=np.inf):
        """Query and update `LAMMPSAtom` nearest neighbors.

        Parameters
        ----------
        n : int, optional
        rc : nonnegative float, optional

        """
        if self._use_kdtree:
            NN_d, NN_i = self.query_atom_tree(n=n, cutoff_radius=rc)
            for i, atom in enumerate(self._atoms):
                NN_atoms = []
                for j, d in enumerate(NN_d[i]):
                    if d < rc:
                        NN_atoms.append(self._atoms[NN_i[i][j]])
                atom.NN = NN_atoms
        nearest_neighbors = []
        for atom in self._atoms:
            nearest_neighbors.append(atom.NN)
        self._nearest_neighbors = nearest_neighbors[:]
        return np.asarray(self._nearest_neighbors)

    @property
    def Ntypes(self):
        """Number of atom types in `LAMMPSAtoms`."""
        return len(self.atomtypes.keys())

    @property
    def q(self):
        """Return the total net charge of `LAMMPSAtoms`."""
        return np.asarray(self.charges).sum()

    @property
    def velocities(self):
        """Return array of `LAMMPSAtom` velocities."""
        velocities = []
        for atom in self._atoms:
            velocities.append(atom.v)
        self._velocities = velocities[:]
        return np.asarray(self._velocities)

    @property
    def NN_number(self):
        return self._NN_number

    @NN_number.setter
    def NN_number(self, value):
        self._NN_number = int(value)

    @property
    def NN_cutoff(self):
        return self._NN_cutoff

    @NN_cutoff.setter
    def NN_cutoff(self, value):
        self._NN_cutoff = value

    def add_atomtype(self, atom):
        """Add atom type to :py:attr:`~sknano.chemistry.LAMMPSAtoms.atomtypes`.

        Parameters
        ----------
        atom : `LAMMPSAtom`
            an intance of a `LAMMPSAtom` object

        """
        if atom.atomtype not in self._atomtypes:
            self._atomtypes[atom.atomtype] = {}
            self._atomtypes[atom.atomtype]['mass'] = atom.m
            self._atomtypes[atom.atomtype]['q'] = atom.q

    def add_atomtypes(self, atomtypes=[]):
        """Add atom type in atomtypes list to atomtype dictionary.
        Parameters
        ----------
        atomtypes : sequence
            a list of `LAMMPSAtom` object instances

        """
        for atom in atomtypes:
            self.add_atomtype(atom)

    def assign_unique_ids(self, starting_id=1):
        """Assign unique ID to each `LAMMPSAtom` in `Atoms`."""
        for i, atom in enumerate(self._atoms, start=starting_id):
            atom.atomID = i

    def filter_atoms(self, filter_atom_ids, invert=False):
        """Filter `LAMMPSAtoms`.

        Parameters
        ----------
        filter_atom_ids : array_like
        invert : bool, optional

        Returns
        -------
        filtered_atoms : `LAMMPSAtoms`

        """
        #if invert:
        #    filtered_atoms = \
        #        LAMMPSAtoms([atom for atom in self.atoms if
        #               atom.atomID not in filter_atom_ids])

        #else:
        #    filtered_atoms = \
        #        LAMMPSAtoms([atom for atom in self.atoms if
        #               atom.atomID in filter_atom_ids])

        filter_indices = \
            np.in1d(self.atom_ids, filter_atom_ids, invert=invert).nonzero()
        filtered_atoms = \
            LAMMPSAtoms(np.asarray(self._atoms)[filter_indices].tolist())
        return filtered_atoms

    def get_atom(self, atomID=None, index=None):
        try:
            return self._atoms[atomID - 1]
        except (TypeError, IndexError):
            try:
                return self._atoms[index]
            except (TypeError, IndexError):
                return None

    def get_filtered_atom_ids(self, filter_atom_ids, invert=False):
        """Return atom ids filtered by list of `filter_atom_ids`.

        Parameters
        ----------
        invert : bool, optional

        Returns
        -------
        ndarray

        """
        atom_ids = self.atom_ids
        filter_indices = \
            np.in1d(atom_ids, filter_atom_ids, invert=invert).nonzero()
        return atom_ids[filter_indices]

    def get_filtered_coords(self, filter_atom_ids, components=None,
                            as_dict=False, invert=False):
        """Return filtered coordinates filtered by filter_atom_ids.

        Parameters
        ----------
        filter_atom_ids : array_like
        components : {None, sequence}, optional
        as_dict : bool, optional
        invert : bool, optional

        Returns
        -------
        filtered_coords : :py:class:`python:~collections.OrderedDict` or
        ndarray

        """
        atom_ids = self.atom_ids
        coords = self.coords
        filter_indices = \
            np.in1d(atom_ids, filter_atom_ids, invert=invert).nonzero()
        filtered_coords = coords[filter_indices]

        if components is None or components == 'r':
            components = ('x', 'y', 'z')
        elif isinstance(components, str):
            components = (components,)

        if as_dict:
            return OrderedDict(zip(
                components, [filtered_coords[:, xyz_axes.index(component)]
                             for component in components]))
        else:
            filtered_coords



# -*- coding: utf-8 -*-
"""
============================================================================
Fullerene structure generators (:mod:`sknano.nanogen._fullerene_generators`)
============================================================================

.. currentmodule:: sknano.nanogen._fullerene_generators

"""
from __future__ import absolute_import, division, print_function
__docformat__ = 'restructuredtext en'

#import copy
#import itertools
#import sys

#import numpy as np

from ..chemistry import Atoms
#from ..tools.refdata import CCbond

from ._fullerenes import Fullerene
from ._structure_generator import StructureGenerator

__all__ = ['FullereneGenerator']


class FullereneGenerator(Fullerene, StructureGenerator):
    u"""Class for generating fullerene structures.

    .. note::

       This class is under development and not functional! Attempting
       to use it raises a `RuntimeError`.

    Parameters
    ----------
    bond : float, optional
        :math:`\\mathrm{a}_{\\mathrm{CC}} =` distance between
        nearest neighbor atoms. Must be in units of **Angstroms**.
    autogen : bool, optional
        if `True`, automatically call
        :py:meth:`~NanotubeGenerator.generate_unit_cell`,
        followed by :py:meth:`~NanotubeGenerator.generate_structure_data`.
    verbose : bool, optional
        if `True`, show verbose output

    Raises
    ------
    `RuntimeError`
        If called. This class is under development and not yet functional.

    Examples
    --------
    First, load the :py:class:`~sknano.nanogen.FullereneGenerator` class.

    >>> from sknano.nanogen import FullereneGenerator
    >>> fg = FullereneGenerator(N=60)

    """

    def __init__(self, N=int, PG=None, autogen=True, verbose=False):

        raise RuntimeError('This class is not yet implemented.')

        super(FullereneGenerator, self).__init__(with_units=False,
                                                 verbose=verbose)

        if autogen:
            self.generate_structure_data()

    def generate_structure_data(self):
        """Generate structure data."""
        self._structure_atoms = Atoms()

    def save_data(self, fname=None, structure_format=None,
                  rotation_angle=None, rot_axis=None, deg2rad=True,
                  center_CM=True):
        """Save structure data.

        See :py:meth:`~sknano.nanogen.StructureGenerator.save_data` method
        for documentation.

        """
        if fname is None:
            fname = 'C{}'.format(self.Natoms)

        super(FullereneGenerator, self).save_data(
            fname=fname, structure_format=structure_format,
            rotation_angle=rotation_angle, rot_axis=rot_axis,
            deg2rad=deg2rad, center_CM=center_CM)



# -*- coding: utf-8 -*-
"""
=======================================================
Math helper functions (:mod:`sknano.tools._mathfuncs`)
=======================================================

.. currentmodule:: sknano.tools._mathfuncs

"""
from __future__ import absolute_import, division, print_function
__docformat__ = 'restructuredtext en'

from fractions import gcd

__all__ = ['totient_func']


def totient_func(n=int):
    """Compute the totatives of :math:`n`.

    Parameters
    ----------
    n : int

    Returns
    -------
    int : the number of totatives of :math:`n`

    """
    if not isinstance(n, int) and int(n) != n:
        raise ValueError('n must be an integer')
    n = int(n)

    tots = 0
    for k in xrange(1, n + 1):
        if gcd(n, k) == 1:
            tots += 1

    return tots



# -*- coding: utf-8 -*-
"""
===================================================================
Structure generator (:mod:`sknano.nanogen._structure_generator`)
===================================================================

.. currentmodule:: sknano.nanogen._structure_generator

"""
from __future__ import absolute_import, division, print_function
__docformat__ = 'restructuredtext en'

import os

from ..chemistry import Atoms
from ..structure_io import DATAWriter, XYZWriter, default_structure_format, \
    supported_structure_formats
from ..tools import rotation_matrix

__all__ = ['StructureGenerator']


class StructureGenerator(object):
    """Base class for structure generator classes."""

    def __init__(self):
        self._fname = None
        self._fpath = None
        self._structure_atoms = Atoms()
        self._structure_format = None
        self._unit_cell = Atoms()

    @property
    def fname(self):
        """Structure file name."""
        return self._fname

    @property
    def fpath(self):
        """Structure file path."""
        return self._fpath

    @property
    def structure_atoms(self):
        """Return structure :class:`~sknano.chemistry.Atoms`."""
        return self._structure_atoms

    @property
    def structure_format(self):
        """Structure file format."""
        return self._structure_format

    @property
    def unit_cell(self):
        """Return unit cell :class:`~sknano.chemistry.Atoms`."""
        return self._unit_cell

    def save_data(self, fname=None, outpath=None, structure_format=None,
                  rotation_angle=None, rot_axis=None, deg2rad=True,
                  center_CM=True):
        """Save structure data.

        Parameters
        ----------
        fname : {None, str}, optional
            file name string
        outpath : str, optional
            Output path for structure data file.
        structure_format : {None, str}, optional
            chemical file format of saved structure data. Must be one of:

                - `xyz`
                - `data`

            If `None`, then guess based on `fname` file extension or
            default to `xyz` format.
        rotation_angle : {None, float}, optional
            Angle of rotation
        rot_axis : {'x', 'y', 'z'}, optional
            Rotation axis
        deg2rad : bool, optional
            Convert `rotation_angle` from degrees to radians.
        center_CM : bool, optional
            Center center-of-mass on origin.

        """
        if fname.endswith(supported_structure_formats) and \
                structure_format is None:
            for ext in supported_structure_formats:
                if fname.endswith(ext):
                    structure_format = ext
                    break
        elif structure_format is None or \
            structure_format not in supported_structure_formats or \
            (not fname.endswith(supported_structure_formats) and
             structure_format not in supported_structure_formats):
            structure_format = default_structure_format

        if not fname.endswith(structure_format):
            fname += '.' + structure_format
        self._fname = fname

        if outpath is not None:
            self._fpath = os.path.join(outpath, fname)

        self._structure_format = structure_format

        if center_CM:
            self.structure_atoms.center_CM()

        if rotation_angle is not None:
            R_matrix = rotation_matrix(rotation_angle,
                                       rot_axis=rot_axis,
                                       deg2rad=deg2rad)
            self.structure_atoms.rotate(R_matrix)

        if structure_format == 'data':
            DATAWriter.write(fname=fname, outpath=outpath,
                             atoms=self.structure_atoms)
        else:
            XYZWriter.write(fname=fname, outpath=outpath,
                            atoms=self.structure_atoms)



# -*- coding: utf-8 -*-
"""
===================================================
NanoGen GUI front-ends (:mod:`sknano.nanogen_gui`)
===================================================

.. currentmodule:: sknano.nanogen_gui

.. versionadded:: 0.2.24

"""
from __future__ import division, print_function, absolute_import
__docformat__ = 'restructuredtext en'

from ._ng_model import *
from ._ng_view import *
from ._ng_controller import *

__all__ = [s for s in dir() if not s.startswith('_')]



# -*- coding: utf-8 -*-
"""
=================================================
Helper functions (:mod:`sknano.analysis._funcs`)
=================================================

.. currentmodule:: sknano.analysis._funcs

"""
from __future__ import division, print_function, absolute_import

__all__ = ['get_nearest_neighbor_counts']


def get_nearest_neighbor_counts(adata=None, datafile=None,
                                filter_elements=None, invert=False):
    pass
#    if adata is not None:
#        atom_data = adata
#    elif datafile is not None and isinstance(datafile, str):
#        atom_data = get_atom_data_asarray(datafile)
#
#    #atom_ids = get_atom_ids_asarray(adata=atom_data)
#
#    filtered_coords = \
#        get_filtered_coords(adata=atom_data, filter_elements=vacIDs)
#    NNtree = spatial.KDTree(filtered_coords)
#    print(NNtree.data)
#
#    # compute things like the distance for which 50% of vacancies
#    # lie within a distance r of each other, etc.
#
#    #return None


def get_nearest_neighbor_comp_array_dict(atom_ids):
    pass
#    nearest_neighbor_comp_dict = OrderedDict()
#    vac_coords = \
#         get_filtered_coords(adata=self.atom_data, filter_elements=vacIDs)
#    #vac_coords = self.get_filtered_coords(vacIDs)
#    for comp in self.position_components:
#        vac_distances = self.get_vac_distances(vac_coords, comp)
#        nearest_neighbor_array = vac_distances[np.where(vac_distances == \
#                                                        vac_distances.min())]
#        nearest_neighbor_comp_dict[comp] = nearest_neighbor_array
#
#    return nearest_neighbor_comp_dict




# THIS FILE IS GENERATED FROM SCIKIT-NANO SETUP.PY
short_version = '0.2.24'
version = '0.2.24'
full_version = '0.2.24'
git_revision = '21db3ce61aa3ba3f8dfd1dd9aa98340bb8772467'
release = True

if not release:
    version = full_version



# -*- coding: utf-8 -*-
"""
=============================================================================
Generate defects in structure data (:mod:`sknano.nanogen._defect_generators`)
=============================================================================

.. currentmodule:: sknano.nanogen._defect_generators

"""
from __future__ import absolute_import, division, print_function
__docformat__ = 'restructuredtext en'

import os

import numpy as np

from ..chemistry import Atoms
from ..structure_io import DATAReader, DATAWriter, XYZWriter, \
    XYZ2DATAConverter, StructureFormatError, supported_structure_formats

vac_type_cluster_size_map = {'double': 2, 'triple': 3}

__all__ = ['DefectGenerator', 'CrossLinkedDefectGenerator',
           'StoneWalesDefectGenerator', 'VacancyGenerator']


class DefectGenerator(object):
    pass


class CrossLinkedDefectGenerator(object):
    pass


class StoneWalesDefectGenerator(object):
    pass


class VacancyGenerator(object):
    """Base class for generating vacancies in structure data.

    Parameters
    ----------
    fname : str
        structure data filename
    outpath : str, optional
        Output path for structure data file.
    structure_format : {None, str}, optional
        chemical file format of saved structure data.
        If `None`, then guess based on `fname` file extension.
        Otherwise, must be one of:

            - `xyz`
            - `data`

    verbose : bool, optional
        Verbose output

    """
    def __init__(self, fname=str, outpath=None, structure_format=None,
                 verbose=False):

        if fname.endswith(supported_structure_formats) and \
                structure_format is None:
            for ext in supported_structure_formats:
                if fname.endswith(ext):
                    structure_format = ext
                    break
        else:
            if (not fname.endswith(supported_structure_formats) and
                    structure_format is None) or \
                    (structure_format is not None and
                        structure_format not in supported_structure_formats):
                raise StructureFormatError(
                    '{} is not a supported structure format'.format(
                        structure_format))

        self._fname = fname
        self._outpath = outpath
        self._structure_format = structure_format
        self._verbose = verbose

        # parse structure data
        if self._structure_format == 'data':
            self._structure_data = DATAReader(fname)
        elif self._structure_format == 'xyz':
            self._structure_data = \
                XYZ2DATAConverter(fname).convert(return_reader=True)

        self._atoms = self._structure_data.atoms
        self._atom_ids = self._atoms.atom_ids
        self._atom_coords = self._atoms.get_coords(as_dict=True)

        self._Nvacs = 0
        self._Nvac_clusters = 0
        self._Nvac_sites = 0

        self._vac_ids = np.empty(0, dtype=int)
        self._vac_type = 'single'
        self._cluster_size = 1

        self._vmd_selection_radius = np.sqrt(10.5)
        self._show_vmd_selection_cmd = True

    @property
    def atoms(self):
        return self._atoms

    @property
    def atom_ids(self):
        return self._atom_ids

    @property
    def atom_coords(self):
        return self._atom_coords

    @property
    def Nvacs(self):
        return self._Nvacs

    @Nvacs.setter
    def Nvacs(self, value=int):
        self._Nvacs = value

    @property
    def Nvac_clusters(self):
        return self._Nvac_clusters

    @Nvac_clusters.setter
    def Nvac_clusters(self, value=int):
        self._Nvac_clusters = value

    @property
    def Nvac_sites(self):
        return self._Nvac_sites

    @Nvac_sites.setter
    def Nvac_sites(self, value=int):
        self._Nvac_sites = value

    @property
    def cluster_size(self):
        return self._cluster_size

    @cluster_size.setter
    def cluster_size(self, value):
        self._cluster_size = value

    @property
    def vac_ids(self):
        return self._vac_ids

    @property
    def vac_type(self):
        return self._vac_type

    @vac_type.setter
    def vac_type(self, value):
        self._vac_type = value

    @property
    def vmd_selection_radius(self):
        return self._vmd_selection_radius

    @vmd_selection_radius.setter
    def vmd_selection_radius(self, value):
        self._vmd_selection_radius = value

    @property
    def show_vmd_selection_cmd(self):
        return self._show_vmd_selection_cmd

    @show_vmd_selection_cmd.setter
    def show_vmd_selection_cmd(self, value=bool):
        self._show_vmd_selection_cmd = value

    def _random_vacancy_generator(self):
        """Generate random vacancies in structure data."""
        self._vac_ids = \
            np.random.choice(self._atom_ids,
                             size=self._Nvac_sites,
                             replace=False)

    def _generate_vmd_selection_cmd(self):

        selection_radius = self._vmd_selection_radius
        selections = []
        for atom in self._removed_atoms:
            selection_cmd = \
                "(((x-{:.4f})^2 + ".format(atom.x) + \
                "(y-{:.4f})^2 + ".format(atom.y) + \
                "(z-{:.4f})^2) <= {:.2f})".format(atom.z,
                                                  selection_radius**2)
            selections.append(selection_cmd)

        vmd_selection_cmd = ' or '.join(selections)
        print('copy and paste the following VMD command to select\n'
              'the atoms surrounding the vacancies:\n\n'
              '{}\n'.format(vmd_selection_cmd))

    def _generate_single_vacancies(self):
        self._removed_atoms = \
            self._atoms.filter_atoms(self._vac_ids, invert=False)

    def _generate_multi_vacancies(self):
        vac_type_properties = {'double': {'cluster_size': 2,
                                          'NN_cutoff': 1.5},
                               'triple': {'cluster_size': 3,
                                          'NN_cutoff': 1.5}}
        vac_props = vac_type_properties[self._vac_type]
        self._cluster_size = vac_props['cluster_size']
        self._atoms.NN_cutoff = vac_props['NN_cutoff']
        self._atoms.update_nearest_neighbors()

        vac_atoms = Atoms()
        for vac_id in self._vac_ids:
            vac_atom = self._atoms.get_atom(atomID=vac_id)
            vac_atoms.append(vac_atom)
            vac_atoms.extend(np.random.choice(vac_atom.NN,
                                              size=self._cluster_size-1,
                                              replace=False).tolist())
        self._removed_atoms = \
            self._atoms.filter_atoms(vac_atoms.atom_ids, invert=False)

    def _generate_stone_wales_vacancies(self):
        pass

    def _generate_vacancy_structure(self):
        """Generate vacancy structure."""
        if self._vac_type in ('double', 'triple'):
            self._generate_multi_vacancies()
        elif self._vac_type in ('stone-wales', 'SW'):
            self._generate_stone_wales_vacancies()
        else:
            self._generate_single_vacancies()

        if self._show_vmd_selection_cmd:
            self._generate_vmd_selection_cmd()

        self._Nvacs = self._removed_atoms.Natoms

        self._remaining_atoms = \
            self._atoms.filter_atoms(self._removed_atoms.atom_ids, invert=True)
        #remaining_atoms.assign_unique_ids()

        self._save_vacancy_structure_data()

    def _generate_output_fname(self):
        self._output_fname = \
            os.path.splitext(os.path.basename(self._fname))[0] + \
            '+{}_vacancies'.format(self._Nvacs)

    def _save_vacancy_structure_data(self):
        self._generate_output_fname()
        DATAWriter.write(fname=self._output_fname, outpath=self._outpath,
                         atoms=self._remaining_atoms,
                         boxbounds=self._structure_data.boxbounds,
                         comment_line=self._structure_data.comment_line)
        XYZWriter.write(fname=self._output_fname, outpath=self._outpath,
                        atoms=self._remaining_atoms,
                        comment_line=self._structure_data.comment_line)



# -*- coding: utf-8 -*-
"""
==============================================================================
Nanotube bundle generators (:mod:`sknano.nanogen._nanotube_bundle_generators`)
==============================================================================

.. currentmodule:: sknano.nanogen._nanotube_bundle_generators

"""
from __future__ import absolute_import, division, print_function
__docformat__ = 'restructuredtext en'

import copy
#import itertools

import numpy as np

from ..chemistry import Atom, Atoms
from ..tools import plural_word_check
from ..tools.refdata import CCbond

from ._nanotubes import Nanotube, NanotubeBundle
from ._nanotube_generators import NanotubeGenerator, MWNTGenerator

__all__ = ['NanotubeBundleGenerator', 'MWNTBundleGenerator']


class NanotubeBundleGenerator(NanotubeGenerator, NanotubeBundle):
    u"""Class for generating nanotube bundles.

    .. versionadded:: 0.2.4

    Parameters
    ----------
    n, m : int
        Chiral indices defining the nanotube chiral vector
        :math:`\\mathbf{C}_{h} = n\\mathbf{a}_{1} + m\\mathbf{a}_{2} = (n, m)`.
    nx, ny, nz : int, optional
        Number of repeat unit cells in the :math:`x, y, z` dimensions.
    element1, element2 : {str, int}, optional
        Element symbol or atomic number of basis
        :class:`~sknano.chemistry.Atoms` 1 and 2
    bond : float, optional
        :math:`\\mathrm{a}_{\\mathrm{CC}} =` distance between
        nearest neighbor atoms. Must be in units of **Angstroms**.
    vdw_spacing : float, optional
        van der Waals distance between nearest neighbor tubes

        .. versionadded:: 0.2.5

    bundle_packing : {None, 'hcp', 'hexagonal', 'ccp', 'cubic'}, optional
        Packing arrangement of nanotubes bundles.
        If `bundle_packing` is `None`, then it will be determined by the
        `bundle_geometry` parameter if `bundle_geometry` is not `None`.
        If both `bundle_packing` and `bundle_geometry` are `None`, then
        `bundle_packing` defaults to `hexagonal`.

        .. versionadded:: 0.2.5

    bundle_geometry : {None, 'triangle', 'hexagon', 'square', 'rectangle',
                       'rhombus', 'rhomboid'}, optional
        Force a specific geometry on the nanotube bundle boundaries.

        .. versionadded:: 0.2.5

    Lx, Ly, Lz : float, optional
        length of bundle in :math:`x, y, z` dimensions in **nanometers**.
        Overrides the :math:`n_x, n_y, n_z` cell values.

        .. versionadded:: 0.2.5

    fix_Lz : bool, optional
        Generate the nanotube with length as close to the specified
        :math:`L_z` as possible. If `True`, then
        non integer :math:`n_z` cells are permitted.

        .. versionadded:: 0.2.6

    autogen : bool, optional
        if `True`, automatically call
        :meth:`~NanotubeGenerator.generate_unit_cell`,
        followed by :meth:`~NanotubeGenerator.generate_structure_data`.
    verbose : bool, optional
        if `True`, show verbose output

    Examples
    --------

    Using the `NanotubeBundleGenerator` class, you can generate structure
    data for nanotube *bundles* with either cubic close packed (ccp) or
    hexagonal close packed (hcp) arrangement of nanotubes. The bundle
    packing arrangement is set using the `bundle_packing` parameter.

    You can also enforce a specific
    `bundle geometry` which will try and build the nanotube bundle such
    that it "fits inside" the boundaries of a specified geometric shape.
    This allows you to generate **hcp** bundles that are trianglar,
    hexagonal, or rectangular in shape, as some of the examples below
    illustrate.

    In general, setting `cubic` bundling will
    generate rectangular bundles (square bundles if :math:`n_x = n_y`)
    and specifying `hcp` bundling will generate *rhomboidal* bundles
    (*i.e.* bundles arranged within a rhomboid) (rhombuses if
    :math:`n_x = n_y`).

    To start, let's generate an hcp bundle of
    :math:`C_{\\mathrm{h}} = (10, 5)` SWCNTs and cell count
    :math:`n_x=10, n_y=3, n_z=5`.

    >>> from sknano.nanogen import NanotubeBundleGenerator
    >>> SWCNTbundle = NanotubeBundleGenerator(n=10, m=5, nx=10, ny=3, nz=5)
    >>> SWCNTbundle.save_data()

    The rendered structure looks like:

    .. image:: /images/1005_hcp_10cellsx3cellsx5cells-01.png

    Now let's generate a nice hexagon bundle, 3 tubes wide, with
    :math:`C_{\\mathrm{h}} = (6, 5)`.

    >>> SWCNTbundle = NanotubeBundleGenerator(n=6, m=5, nz=5,
    ...                                       bundle_geometry='hexagon')
    >>> SWCNTbundle.save_data()

    which looks like:

    .. image:: /images/0605_hcp_7tube_hexagon-01.png

    Remember, all of the :meth:`~NanotubeBundleGenerator.save_data`
    methods allow you to rotate the structure data before saving:

    >>> SWCNTbundle.save_data(fname='0605_hcp_7tube_hexagon_rot-30deg.xyz',
    ...                       rot_axis='z', rotation_angle=30)

    .. image:: /images/0605_hcp_7tube_hexagon_rot-30deg-01.png

    Now, just because we can, let's make a big ass hexagon bundle with
    :math:`C_{\\mathrm{h}} = (10, 0)`.

    >>> BIGASSHEXABUN = NanotubeBundleGenerator(10, 0, nx=25, ny=25, nz=1,
    ...                                         bundle_geometry='hexagon')
    >>> BIGASSHEXABUN.save_data()

    Take a look at the 469 :math:`(10, 0)` unit cells in this big ass bundle!

    .. image:: /images/1000_hcp_469tube_hexagon-01.png

    Lastly, here's a look at a bundle generated with cubic close packed
    bundle arrangement:

    >>> SWCNTbundle = NanotubeBundleGenerator(10, 10, nx=3, ny=3, nz=5,
    ...                                       bundle_packing='cubic')
    >>> SWCNTbundle.save_data()

    The rendered `ccp` structure looks like:

    .. image:: /images/1010_ccp_3cellsx3cellsx5cells-01.png

    """

    def __init__(self, n=int, m=int, nx=1, ny=1, nz=1,
                 element1='C', element2='C', bond=CCbond, vdw_spacing=3.4,
                 bundle_packing=None, bundle_geometry=None, Lx=None, Ly=None,
                 Lz=None, fix_Lz=False, with_units=False, units=None,
                 autogen=True, verbose=False):

        super(NanotubeBundleGenerator, self).__init__(
            n=n, m=m, nx=nx, ny=ny, nz=nz, element1=element1,
            element2=element2, Lx=Lx, Ly=Ly, Lz=Lz, fix_Lz=fix_Lz, bond=bond,
            with_units=with_units, units=units, autogen=False, verbose=verbose)

        self.compute_bundle_params()

        self._r1 = np.zeros(3)
        self._r1[0] = \
            Nanotube.compute_dt(n=n, m=m, bond=bond, with_units=False) + \
            vdw_spacing

        self._r2 = np.zeros(3)

        if bundle_packing is None and \
                bundle_geometry in ('square', 'rectangle'):
            bundle_packing = 'cubic'
        elif bundle_packing is None:
            bundle_packing = 'hexagonal'
        elif (bundle_packing in ('cubic', 'ccp') and bundle_geometry not in
                (None, 'square', 'rectangle')) or \
                (bundle_packing in ('hexagonal', 'hcp') and bundle_geometry
                 not in (None, 'triangle', 'hexagon', 'rhombus', 'rhomboid')):
            bundle_geometry = None

        if bundle_packing in ('cubic', 'ccp'):
            self._r2[1] = self._r1[0]
        else:
            self._r2[0] = self._r1[0] * np.cos(2 * np.pi / 3)
            self._r2[1] = self._r1[0] * np.sin(2 * np.pi / 3)

        #self._r1[2] = self._r2[2] = self.T

        self._bundle_packing = bundle_packing
        self._bundle_geometry = bundle_geometry

        if autogen:
            super(NanotubeBundleGenerator, self).generate_unit_cell()
            self.generate_structure_data()

    def generate_structure_data(self):
        """Generate structure data."""
        super(NanotubeBundleGenerator, self).generate_structure_data()

        self._Ntubes = 0

        swnt0 = copy.deepcopy(self._structure_atoms)
        self._structure_atoms = Atoms()
        if self._bundle_geometry == 'hexagon':
            nrows = max(self._nx, self._ny, 3)
            if nrows % 2 != 1:
                nrows += 1

            ntubes_per_end_rows = int((nrows + 1) / 2)

            row = 0
            ntubes_per_row = nrows
            while ntubes_per_row >= ntubes_per_end_rows:
                if row == 0:
                    for n in xrange(ntubes_per_row):
                        swnt = Atoms(atoms=swnt0, deepcopy=True)
                        swnt.center_CM()
                        dr = n * self._r1
                        swnt.translate(dr)
                        self._structure_atoms.extend(swnt.atoms)
                        self._Ntubes += 1
                else:
                    for nx in xrange(ntubes_per_row):
                        for ny in (-row, row):
                            swnt = Atoms(atoms=swnt0, deepcopy=True)
                            swnt.center_CM()
                            dr = np.zeros(3)
                            dr[0] = abs(ny * self._r2[0])
                            dr[1] = ny * self._r2[1]
                            dr = nx * self._r1 + dr
                            swnt.translate(dr)
                            self._structure_atoms.extend(swnt.atoms)
                            self._Ntubes += 1
                row += 1
                ntubes_per_row = nrows - row
        else:
            for nx in xrange(self._nx):
                for ny in xrange(self._ny):
                    swnt = Atoms(atoms=swnt0, deepcopy=True)
                    swnt.center_CM()
                    dr = nx * self._r1 + ny * self._r2
                    swnt.translate(dr)
                    self._structure_atoms.extend(swnt.atoms)
                    self._Ntubes += 1
        self._Natoms_per_bundle = \
            self.compute_Natoms_per_bundle(n=self._n, m=self._m,
                                           nz=self._nz,
                                           Ntubes=self._Ntubes)
        #self.update_structure_atoms_attributes()

    def save_data(self, fname=None, outpath=None, structure_format=None,
                  rotation_angle=None, rot_axis=None, deg2rad=True,
                  center_CM=True):
        """Save structure data.

        See :meth:`~sknano.nanogen.StructureGenerator.save_data` method
        for documentation.

        """
        if fname is None:
            chirality = '{}{}'.format('{}'.format(self._n).zfill(2),
                                      '{}'.format(self._m).zfill(2))
            packing = '{}cp'.format(self._bundle_packing[0])
            #Ntubes = ''.join(('{}'.format(self._Ntubes),
            #                  plural_word_check('tube', self._Ntubes)))
            Ntube = '{}tube'.format(self._Ntubes)

            fname_wordlist = None
            if self._bundle_geometry is None:
                nx = ''.join(('{}'.format(self._nx),
                             plural_word_check('cell', self._nx)))
                ny = ''.join(('{}'.format(self._ny),
                             plural_word_check('cell', self._ny)))
                if self._assume_integer_unit_cells:
                    nz = ''.join(('{}'.format(self._nz),
                                  plural_word_check('cell', self._nz)))
                else:
                    nz = ''.join(('{:.2f}'.format(self._nz),
                                  plural_word_check('cell', self._nz)))
                cells = 'x'.join((nx, ny, nz))
                fname_wordlist = (chirality, packing, cells)
            else:
                fname_wordlist = \
                    (chirality, packing, Ntube, self._bundle_geometry)

            fname = '_'.join(fname_wordlist)

        super(NanotubeBundleGenerator, self).save_data(
            fname=fname, outpath=outpath, structure_format=structure_format,
            rotation_angle=rotation_angle, rot_axis=rot_axis,
            deg2rad=deg2rad, center_CM=center_CM)


class MWNTBundleGenerator(MWNTGenerator, NanotubeBundle):
    u"""Class for generating multi-walled nanotube bundles.

    .. versionadded:: 0.2.20

    Parameters
    ----------
    n, m : int
        Chiral indices defining the nanotube chiral vector
        :math:`\\mathbf{C}_{h} = n\\mathbf{a}_{1} + m\\mathbf{a}_{2} = (n, m)`.
    nx, ny, nz : int, optional
        Number of repeat unit cells in the :math:`x, y, z` dimensions.
    element1, element2 : {str, int}, optional
        Element symbol or atomic number of basis
        :class:`~sknano.chemistry.Atoms` 1 and 2
    bond : float, optional
        :math:`\\mathrm{a}_{\\mathrm{CC}} =` distance between
        nearest neighbor atoms. Must be in units of **Angstroms**.
    Lx, Ly, Lz : float, optional
        length of bundle in :math:`x, y, z` dimensions in **nanometers**.
        Overrides the :math:`n_x, n_y, n_z` cell values.
    fix_Lz : bool, optional
        Generate the nanotube with length as close to the specified
        :math:`L_z` as possible. If `True`, then
        non integer :math:`n_z` cells are permitted.
    add_outer_shells : bool, optional
        Build the MWNT by adding outer shells

        .. versionadded:: 0.2.23

    add_inner_shells : bool, optional
        Build the MWNT by adding inner shells

        .. versionadded:: 0.2.23

    max_shells : int, optional
        Maximum number of shells per MWNT.
    max_shell_diameter : float, optional
        Maximum shell diameter, in units of **Angstroms**.
    min_shells : int, optional
        Minimum number of shells per MWNT.
    min_shell_diameter : float, optional
        Minimum shell diameter, in units of **Angstroms**.
    new_shell_type : {None, 'AC', 'ZZ', 'achiral', 'chiral'}, optional
        If `None`, the chiralities of the new shells are constrained only
        by their diameter and will be chosen randomly if more than one
        candidate chirality exists. If not `None`, then the
        `new_shell_type` will be added as a constraint.
    shell_spacing : float, optional
        Shell spacing in units of **Angstroms**. Default
        value is the van der Waals interaction distance of 3.4 Angstroms.
    vdw_spacing : float, optional
        van der Waals distance between nearest neighbor tubes
    bundle_packing : {None, 'hcp', 'hexagonal', 'ccp', 'cubic'}, optional
        Packing arrangement of MWNT bundles.  If `bundle_packing` is `None`,
        then it will be determined by the `bundle_geometry` parameter if
        `bundle_geometry` is not `None`.  If both `bundle_packing` and
        `bundle_geometry` are `None`, then `bundle_packing` defaults to
        `hexagonal`.
    bundle_geometry : {None, 'triangle', 'hexagon', 'square', 'rectangle',
                       'rhombus', 'rhomboid'}, optional
    autogen : bool, optional
        if `True`, automatically call
        :meth:`~MWNTGenerator.generate_unit_cell`,
        followed by :meth:`~MWNTGenerator.generate_structure_data`.
    verbose : bool, optional
        if `True`, show verbose output

    Examples
    --------

    >>> from sknano.nanogen import MWNTBundleGenerator
    >>> mwntbundle = MWNTBundleGenerator(n=40, m=40, max_shells=5,
    ...                                  Lz=1.0, fix_Lz=True,
    ...                                  bundle_geometry='hexagon')
    >>> mwntbundle.save_data()

    .. image:: /images/5shell_mwnt_4040_outer_Ch_hcp_7tube_hexagon-01.png

    """
    def __init__(self, n=int, m=int, nx=1, ny=1, nz=1,
                 element1='C', element2='C', bond=CCbond,
                 Lx=None, Ly=None, Lz=None, fix_Lz=False,
                 add_inner_shells=True, add_outer_shells=False,
                 max_shells=None, max_shell_diameter=np.inf,
                 min_shells=None, min_shell_diameter=0.0,
                 new_shell_type=None, shell_spacing=3.4,
                 vdw_spacing=3.4, bundle_packing=None, bundle_geometry=None,
                 with_units=False, units=None, autogen=True, verbose=False):

        super(MWNTBundleGenerator, self).__init__(
            n=n, m=m, nx=nx, ny=ny, nz=nz,
            element1=element1, element2=element2, bond=bond,
            Lx=Lx, Ly=Ly, Lz=Lz, fix_Lz=fix_Lz,
            add_inner_shells=add_inner_shells,
            add_outer_shells=add_outer_shells,
            max_shells=max_shells, max_shell_diameter=max_shell_diameter,
            min_shells=min_shells, min_shell_diameter=min_shell_diameter,
            new_shell_type=new_shell_type, shell_spacing=shell_spacing,
            with_units=with_units, units=units, autogen=False, verbose=verbose)

        self.compute_bundle_params()

        self._r1 = np.zeros(3)
        self._r1[0] = Nanotube.compute_dt(n=n, m=m, bond=bond) + \
            vdw_spacing

        self._r2 = np.zeros(3)

        if bundle_packing is None and \
                bundle_geometry in ('square', 'rectangle'):
            bundle_packing = 'cubic'
        elif bundle_packing is None:
            bundle_packing = 'hexagonal'
        elif (bundle_packing in ('cubic', 'ccp') and bundle_geometry not in
                (None, 'square', 'rectangle')) or \
                (bundle_packing in ('hexagonal', 'hcp') and bundle_geometry
                 not in (None, 'triangle', 'hexagon', 'rhombus', 'rhomboid')):
            bundle_geometry = None

        if bundle_packing in ('cubic', 'ccp'):
            self._r2[1] = self._r1[0]
        else:
            self._r2[0] = self._r1[0] * np.cos(2 * np.pi / 3)
            self._r2[1] = self._r1[0] * np.sin(2 * np.pi / 3)

        self._bundle_packing = bundle_packing
        self._bundle_geometry = bundle_geometry

        if autogen:
            super(MWNTBundleGenerator, self).generate_unit_cell()
            self.generate_structure_data()

    def _generate_unit_cell(self, n=int, m=int):
        """Generate the unit cell of a MWNT shell"""
        eps = 0.01
        bond = self._bond
        e1 = self._element1
        e2 = self._element2

        N = Nanotube.compute_N(n=n, m=m)
        aCh = Nanotube.compute_chiral_angle(n=n, m=m, rad2deg=False)
        rt = Nanotube.compute_rt(n=n, m=m, bond=bond, with_units=False)
        T = Nanotube.compute_T(n=n, m=m, bond=bond, with_units=False)

        tau = Nanotube.compute_tau(n=n, m=m, bond=bond, with_units=False)
        dtau = bond * np.sin(np.pi / 6 - aCh)

        psi = Nanotube.compute_psi(n=n, m=m)
        dpsi = bond * np.cos(np.pi / 6 - aCh) / rt

        unit_cell = Atoms()

        for i in xrange(1, N + 1):
            x1 = rt * np.cos(i * psi)
            y1 = rt * np.sin(i * psi)
            z1 = i * tau

            while z1 > T - eps:
                z1 -= T

            atom1 = Atom(e1, x=x1, y=y1, z=z1)
            atom1.rezero_coords()

            unit_cell.append(atom1)

            x2 = rt * np.cos(i * psi + dpsi)
            y2 = rt * np.sin(i * psi + dpsi)
            z2 = i * tau - dtau
            while z2 > T - eps:
                z2 -= T

            atom2 = Atom(e2, x=x2, y=y2, z=z2)
            atom2.rezero_coords()

            unit_cell.append(atom2)

        return unit_cell

    def generate_structure_data(self):
        """Generate structure data."""
        super(MWNTBundleGenerator, self).generate_structure_data()

        self._Ntubes = 0

        mwnt0 = copy.deepcopy(self._structure_atoms)
        self._structure_atoms = Atoms()

        if self._bundle_geometry == 'hexagon':
            nrows = max(self._nx, self._ny, 3)
            if nrows % 2 != 1:
                nrows += 1

            ntubes_per_end_rows = int((nrows + 1) / 2)

            row = 0
            ntubes_per_row = nrows
            while ntubes_per_row >= ntubes_per_end_rows:
                if row == 0:
                    for n in xrange(ntubes_per_row):
                        mwnt = Atoms(atoms=mwnt0.atoms, deepcopy=True)
                        mwnt.center_CM()
                        dr = n * self._r1
                        mwnt.translate(dr)
                        self._structure_atoms.extend(mwnt.atoms)
                        self._Ntubes += 1
                else:
                    for nx in xrange(ntubes_per_row):
                        for ny in (-row, row):
                            mwnt = Atoms(atoms=mwnt0.atoms, deepcopy=True)
                            mwnt.center_CM()
                            dr = np.zeros(3)
                            dr[0] = abs(ny * self._r2[0])
                            dr[1] = ny * self._r2[1]
                            dr = nx * self._r1 + dr
                            mwnt.translate(dr)
                            self._structure_atoms.extend(mwnt.atoms)
                            self._Ntubes += 1
                row += 1
                ntubes_per_row = nrows - row
        else:
            for nx in xrange(self._nx):
                for ny in xrange(self._ny):
                    mwnt = Atoms(atoms=mwnt0.atoms, deepcopy=True)
                    mwnt.center_CM()
                    dr = nx * self._r1 + ny * self._r2
                    mwnt.translate(dr)
                    self._structure_atoms.extend(mwnt.atoms)
                    self._Ntubes += 1
        self._Natoms_per_bundle = self._Ntubes * self._Natoms_per_tube

        if self._verbose:
            print('Ntubes: {}'.format(self._Ntubes))
            print('Natoms_per_bundle: {}'.format(self._Natoms_per_bundle))

    def save_data(self, fname=None, outpath=None, structure_format=None,
                  rotation_angle=None, rot_axis=None, deg2rad=True,
                  center_CM=True):
        """Save structure data.

        See :meth:`~sknano.nanogen.StructureGenerator.save_data` method
        for documentation.

        """
        if fname is None:
            Nshells = '{}shell_mwnt'.format(self._Nshells_per_tube)
            chirality = '{}{}_{}_Ch'.format('{}'.format(self._n).zfill(2),
                                            '{}'.format(self._m).zfill(2),
                                            self._starting_shell_position)
            packing = '{}cp'.format(self._bundle_packing[0])
            Ntubes = '{}tube'.format(self._Ntubes)

            fname_wordlist = None
            if self._bundle_geometry is None:
                nx = ''.join(('{}'.format(self._nx),
                             plural_word_check('cell', self._nx)))
                ny = ''.join(('{}'.format(self._ny),
                             plural_word_check('cell', self._ny)))
                if self._assume_integer_unit_cells:
                    nz = ''.join(('{}'.format(self._nz),
                                  plural_word_check('cell', self._nz)))
                else:
                    nz = ''.join(('{:.2f}'.format(self._nz),
                                  plural_word_check('cell', self._nz)))
                cells = 'x'.join((nx, ny, nz))

                if self._nx == 1 and self._ny == 1:
                    fname_wordlist = (Nshells, chirality, cells)
                else:
                    fname_wordlist = (Nshells, chirality, packing, cells)
            else:
                fname_wordlist = \
                    (Nshells, chirality, packing, Ntubes,
                     self._bundle_geometry)

            fname = '_'.join(fname_wordlist)

        super(MWNTBundleGenerator, self).save_data(
            fname=fname, outpath=outpath, structure_format=structure_format,
            rotation_angle=rotation_angle, rot_axis=rot_axis,
            deg2rad=deg2rad, center_CM=center_CM)
